
/*
Copyright 2007-2009 Kim A. Brandt <kimabrandt@gmx.de>

This file is part of yarip.

Yarip is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Yarip is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with yarip; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

const CLASS_ID = Components.ID("{46d09504-6a0b-40c8-8ce8-d07cf370d25e}");
const CLASS_NAME = "Yet Another Remove It Permanently";
const CONTRACT_ID = "@yarip.mozdev.org;1";

const prefBranch = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);

const TYPE_DOCUMENT = Components.interfaces.nsIContentPolicy.TYPE_DOCUMENT;
const DOCUMENT_NODE = Components.interfaces.nsIDOMNode.DOCUMENT_NODE;
const ACCEPT = Components.interfaces.nsIContentPolicy.ACCEPT;
const REJECT_SERVER = Components.interfaces.nsIContentPolicy.REJECT_SERVER;

var yarip = {
    STYLE: "extensions/yarip@mozdev.org/style.css",
    XPATH: "yarip.xml",

    PAGE_MODE: 0,
    FQDN_MODE: 1,
    SLD_MODE: 2,
    FIRSTDIR_MODE: 3,
//    USER_MODE: 3,

    TYPE_WHITELIST: 0,
    TYPE_BLACKLIST: 1,
    TYPE_ATTRIBUTE: 2,

    NO_CHANGE: 0,
    MINOR_CHANGE: 1,
    ESSENTIAL_CHANGE: 2,

    BOOLEAN: 0,
    INTEGER: 1,
    STRING: 2,
    RESET: 3,

    INCREMENT_NOT_FOUND: 0,
    INCREMENT_IGNORED: 1,
    INCREMENT_FOUND: 2,

    FIRSTRUN: "extensions.yarip.firstrun",
    VERSION: "extensions.yarip.version",
    ENABLED_ROOT: "extensions.yarip.enabled.",
    ENABLED: "extensions.yarip.enabled.value",
    FLICKER_ROOT: "extensions.yarip.noFlicker.",
    FLICKER: "extensions.yarip.noFlicker.value",
    MODE_ROOT: "extensions.yarip.mode.",
    MODE: "extensions.yarip.mode.value",
    INDEX_ROOT: "extensions.yarip.alwaysUseIndex.",
    INDEX: "extensions.yarip.alwaysUseIndex.value",
    ELEMENTS_ROOT: "extensions.yarip.elementsInContext.",
    ELEMENTS: "extensions.yarip.elementsInContext.value",
    EXCLUSIVE_ROOT: "extensions.yarip.exclusiveOnCreation.",
    EXCLUSIVE: "extensions.yarip.exclusiveOnCreation.value",
    TEMPLATES_ROOT: "extensions.yarip.templatesList.",
    TEMPLATES: "extensions.yarip.templatesList.value",
    PRIVATE_ROOT: "extensions.yarip.privateBrowsing.",
    PRIVATE: "extensions.yarip.privateBrowsing.value",
    PURGE_ROOT: "extensions.yarip.purgeInnerHTML.",
    PURGE: "extensions.yarip.purgeInnerHTML.value",
    SCHEMES_ROOT: "extensions.yarip.schemesList.",
    SCHEMES: "extensions.yarip.schemesList.value",

    VALUE: "value",

    CHARSET: "UTF-8",

    firstLoad: true,
    enabled: null,
    noFlicker: null,
    mode: null,
//    deep: null,

//    monitorDialog: null,
    monitorDialog: new Array(),
    pageDialog: null,

    alwaysUseIndex: false,
    elementsInContext: 4,
    changeType: 0,
    purgeInnerHTML: false,
    exclusiveOnCreation: false,
    templates: [],
    schemesRegExp: null,

//    address: null,
    map: null,
    undoObj: {},
    gBrowser: null,
    mTabContainer: null
}

yarip.OFFICIAL_SCHEME = "(aaas|aaa|acap|afs|cap|cid|crid|data|dav|dict|dns|dtn|fax|file|ftp|gopher|go|h323|https|http|iax2|icap|imap|im|info|ipp|iris\\.beep|iris\\.lwz|iris\\.xpcs|iris\\.xpc|iris|ldap|mailserver|mailto|mid|modem|msrps|msrp|mtqp|mupdate|news|nfs|nntp|opaquelocktoken|pack|pop|pres|prospero|rtsp|service|shttp|sips|sip|snmp|soap\\.beeps|soap\\.beep|tag|telnet|tel|tftp|thismessage|tip|tn3270|tv|urn|vemmi|wais|xmlrpc\\.beeps|xmlrpc\\.beep|xmpp|z39\\.50r|z39\\.50s)";
yarip.UNOFFICIAL_SCHEME = "(about|addbook|afp|aim|applescript|bcp|bk|btspp|callto|castanet|cdv|chrome|chttp|cvs|daytime|device|doi|ed2k|eid|enp|feed|finger|fish|freenet|gg|gizmoproject|gsiftp|gsm-sms|h324|hdl|hnews|httpsy|iioploc|ilu|IOR|ircs|irc|itms|jar|javascript|jdbc|klik|kn|lastfm|ldaps|lifn|livescript|lrq|magnet|mailbox|man|md5|mms|mocha|moz-abmdbdirectory|msnim|myim|notes|nsfw|oai|pcast|phone|php|pop3|printer|psyc|pyimp|rdar|res|rsync|rvp|rwhois|rx|sdp|secondlife|sftp|skype|smb|sms|snews|soap\\.udp|soldat|ssh|steam|SubEthaEdit|svn\\+ssh|svn|t120|tann|tcp|telephone|txmt|uddi|unreal|ut2004|uuid|videotex|view-source|wcap|webcal|whodp|whois\\+\\+|whois|wpn|wtai|xeerkat|xfire|ymsgr)";
yarip.SCHEME = "(" + yarip.OFFICIAL_SCHEME + "|" + yarip.UNOFFICIAL_SCHEME + "):";
yarip.IP_LITERAL = "(\\[((([a-f\\d]{1,4}:){6}([a-f\\d]{1,4}:[a-f\\d]{1,4})|((([1-9]?\\d|1\\d{2}|2[0-4]\\d|25[0-5])\\.){3}([1-9]?\\d|1\\d{2}|2[0-4]\\d|25[0-5])))|(::([a-f\\d]{1,4}:){5}([a-f\\d]{1,4}:[a-f\\d]{1,4})|((([1-9]?\\d|1\\d{2}|2[0-4]\\d|25[0-5])\\.){3}([1-9]?\\d|1\\d{2}|2[0-4]\\d|25[0-5])))|(([a-f\\d]{1,4})?::([a-f\\d]{1,4}:){4}([a-f\\d]{1,4}:[a-f\\d]{1,4})|((([1-9]?\\d|1\\d{2}|2[0-4]\\d|25[0-5])\\.){3}([1-9]?\\d|1\\d{2}|2[0-4]\\d|25[0-5])))|((([a-f\\d]{1,4}:)?[a-f\\d]{1,4})?::([a-f\\d]{1,4}:){3}([a-f\\d]{1,4}:[a-f\\d]{1,4})|((([1-9]?\\d|1\\d{2}|2[0-4]\\d|25[0-5])\\.){3}([1-9]?\\d|1\\d{2}|2[0-4]\\d|25[0-5])))|(((([a-f\\d]{1,4}:){1,2})?[a-f\\d]{1,4})?::([a-f\\d]{1,4}:){2}([a-f\\d]{1,4}:[a-f\\d]{1,4})|((([1-9]?\\d|1\\d{2}|2[0-4]\\d|25[0-5])\\.){3}([1-9]?\\d|1\\d{2}|2[0-4]\\d|25[0-5])))|(((([a-f\\d]{1,4}:){1,3})?[a-f\\d]{1,4})?::([a-f\\d]{1,4}:[a-f\\d]{1,4})|((([1-9]?\\d|1\\d{2}|2[0-4]\\d|25[0-5])\\.){3}([1-9]?\\d|1\\d{2}|2[0-4]\\d|25[0-5])))|(((([a-f\\d]{1,4}:){1,4})?[a-f\\d]{1,4})?::([a-f\\d]{1,4}:[a-f\\d]{1,4})|((([1-9]?\\d|1\\d{2}|2[0-4]\\d|25[0-5])\\.){3}([1-9]?\\d|1\\d{2}|2[0-4]\\d|25[0-5])))|(((([a-f\\d]{1,4}:){1,5})?[a-f\\d]{1,4})?::[a-f\\d]{1,4})|(((([a-f\\d]{1,4}:){1,6})?[a-f\\d]{1,4})?::))\\])";
yarip.IPV4ADDRESS = "((([1-9]?\\d|1\\d{2}|2[0-4]\\d|25[0-5])\\.){3}([1-9]?\\d|1\\d{2}|2[0-4]\\d|25[0-5]))";
yarip.IP = "(" + yarip.IPV4ADDRESS + "|" + yarip.IP_LITERAL + ")";
yarip.AUTHORITY = "([a-z\\d]([\\w-.~!$&*+,=:]|%[a-f\\d]{2})*@)?";
yarip.REG_NAME = "([a-z\\d]([\\w-~!$&*+,=@]|%[a-f\\d]{2})+\\.)+";
yarip.SIMPLE = "([a-z\\d]([\\w-~!$&'()*+,;=]|%[a-f\\d]{2})+)";
//yarip.ULD = "[a-z\\d]([\\w-.~!$&'()*+,;=]|%[a-f\\d]{2})+\\.";
yarip.SLD = yarip.SIMPLE + "\\.";
yarip.TLD = "(ac|ad|aero|ae|af|ag|ai|al|am|an|ao|aq|arpa|ar|asia|as|at|au|aw|ax|az|ba|bb|bd|be|bf|bg|bh|biz|bi|bj|bm|bn|bo|br|bs|bt|bv|bw|by|bz|cat|ca|cc|cd|cf|cg|ch|ci|ck|cl|cm|cn|coop|com|co|cr|cu|cv|cx|cy|cz|de|dj|dk|dm|do|dz|ec|edu|ee|eg|er|es|et|eu|fi|fj|fk|fm|fo|fr|ga|gb|gd|ge|gf|gg|gh|gi|gl|gm|gn|gov|gp|gq|gr|gs|gt|gu|gw|gy|hk|hm|hn|hr|ht|hu|id|ie|il|im|info|int|in|io|iq|ir|is|it|je|jm|jobs|jo|jp|ke|kg|kh|ki|km|kn|kp|kr|kw|ky|kz|la|lb|lc|li|lk|lr|ls|lt|lu|lv|ly|ma|mc|md|me|mg|mh|mil|mk|ml|mm|mn|mobi|mo|mp|mq|mr|ms|mt|museum|mu|mv|mw|mx|my|mz|name|na|nc|net|ne|nf|ng|ni|nl|no|np|nr|nu|nz|om|org|pa|pe|pf|pg|ph|pk|pl|pm|pn|pro|pr|ps|pt|pw|py|qa|re|ro|rs|ru|rw|sa|sb|sc|sd|se|sg|sh|si|sj|sk|sl|sm|sn|so|sr|st|su|sv|sy|sz|tc|td|tel|tf|tg|th|tj|tk|tl|tm|tn|to|tp|travel|tr|tt|tv|tw|tz|ua|ug|uk|um|us|uy|uz|va|vc|ve|vg|vi|vn|vu|wf|ws|xn--0zwm56d|xn--11b5bs3a9aj6g|xn--80akhbyknj4f|xn--9t4b11yi5a|xn--deba0ad|xn--g6w251d|xn--hgbk6aj7f53bba|xn--hlcj6aya9esc7a|xn--jxalpdlp|xn--kgbechtv|xn--zckzah|ye|yt|yu|za|zm|zw)";
yarip.PORT = "(:\\d+)?";
yarip.HIER_PART = "(((\\/\\/)?((([\\w-.~!$&'()*+,;=:]|%[a-f\\d]{2})*@)?(" + yarip.IP + "|([\\w-.~!$&'()*+,;=]|%[a-f\\d]{2})*)" + yarip.PORT + "))([\\w-.~!$&'()*+,;=:@/]|%[a-f\\d]{2})*)";
yarip.PATH = "([;/]([\\w-.~!$&'()*+,;=:@/]|%[a-f\\d]{2})*)?";
yarip.QUERY_FRAGMENT = "([&?]([\\w-.~!$&'()*+,;=:@/?]|%[a-f\\d]{2})*)?(#([\\w-.~!$&'()*+,;=:@/?]|%[a-f\\d]{2})*)?";

yarip.URI1 = "(" + yarip.SCHEME + yarip.HIER_PART + yarip.QUERY_FRAGMENT + ")";
yarip.URI2 = "(" + yarip.AUTHORITY + "(" + yarip.IP + "|(" + yarip.REG_NAME + yarip.TLD + "\\b))" + yarip.PORT + yarip.PATH + yarip.QUERY_FRAGMENT + ")";

//yarip.uriRegExp = new RegExp("^(" + yarip.URI1 + "|" + yarip.URI2 + ")$", "i");
yarip.uriRegExp = new RegExp("^(" + yarip.URI1 + "|" + yarip.URI2 + ")$", "ig");
yarip.noSchemeRegExp = new RegExp("^" + yarip.URI2 + "$", "i");
yarip.simpleRegExp = new RegExp("^" + yarip.SIMPLE + "\\b" + yarip.PORT + "$", "i");
//yarip.domainRegExp = new RegExp("^(" + yarip.IP + "|(" + yarip.ULD + yarip.TLD + "\\b))" + yarip.PORT + "$", "i");
//yarip.findDomainRegExp = new RegExp("\\b(" + yarip.IP + "|(" + yarip.ULD + yarip.TLD + ")|" + yarip.SIMPLE + ")\\b" + yarip.PORT, "i");
yarip.findSldRegExp = new RegExp("\\b(" + yarip.IP + "|(" + yarip.SLD + yarip.TLD + ")|" + yarip.SIMPLE + ")\\b" + yarip.PORT + "$", "i");

yarip.outlineRegExp = /^outline-color:\ rgb\((0|153),\ 0,\ (0|153)\);\ outline-style:\ solid;\ outline-width:\ 3px;$/;
yarip.endOutlineRegExp = /outline-color:\ rgb\((0|153),\ 0,\ (0|153)\);\ outline-style:\ solid;\ outline-width:\ 3px;$/;

yarip.toggleEnabled = function(enabled)
{
    if (enabled == this.enabled) return false;

    if (enabled != true && enabled != false) enabled = !this.enabled;
    this.enabled = enabled;
    this.save();

    if (enabled) this.toggleNoFlicker(this.getValue(this.FLICKER, false, this.BOOLEAN), null, true);
    else this.toggleNoFlicker(false, true, true);

    this.setValue(this.ENABLED, enabled, this.BOOLEAN); // notify
    return true;
}

yarip.toggleNoFlicker = function(noFlicker, dontNotify, force)
{
    if (noFlicker == this.noFlicker && !force) return false;

//    if (noFlicker != true && noFlicker != false) noFlicker = !this.noFlicker;
    if (noFlicker != true && noFlicker != false) noFlicker = !this.getValue(this.FLICKER, false, this.BOOLEAN);
    this.noFlicker = noFlicker;

    this.toggleStyle();
    this.injectCSS("chrome://yarip/content/noflicker.css", this.noFlicker && this.enabled);
    if (!dontNotify) this.setValue(this.FLICKER, this.noFlicker, this.BOOLEAN); // notify
    return true;
}

yarip.toggleStyle = function()
{
    var file = this.getFile(this.STYLE);
    if (file) this.injectCSS("file://" + file.path, this.noFlicker && this.enabled);
}

yarip.setMode = function(mode)
{
    var mode = Number(mode);
//    if (mode != this.USER_MODE && (/*mode == this.mode ||*/ typeof mode != "number" || mode + "" == "NaN")) return false;
    if (typeof mode != "number" || mode + "" == "NaN") return false;
    this.mode = mode;
    this.setValue(this.MODE, mode, this.INTEGER); // notify
    return true;
}

//yarip.toggleDeep = function(deep)
//{
//    if (deep == this.deep) return false;

//    if (deep != true && deep != false) deep = !this.deep;
//    this.deep = deep;

//    this.setValue(this.DEEP, deep, this.BOOLEAN); // notify
//    return true;
//}

yarip.setAlwaysUseIndex = function(value)
{
    if (value == true || value == false) this.alwaysUseIndex = value;
}

yarip.setElementsInContext = function(value)
{
    var value = Number(value);
    if (typeof value != "number" || value + "" == "NaN") return;
    if (value > 0 && value <= 20) this.elementsInContext = value;
}

yarip.setPurgeInnerHTML = function(value)
{
    if (value == true || value == false) this.purgeInnerHTML = value;
}

yarip.setExclusiveOnCreation = function(value)
{
    if (value == true || value == false) this.exclusiveOnCreation = value;
}

yarip.setTemplates = function(value)
{
    this.templates = [];
    if (!value) return;

    this.templates = value.split(" ", 50); // 50 = max templates
}

yarip.setSchemes = function(value)
{
    try {
        this.schemesRegExp = new RegExp(value);
    } catch (e) {
    }
}

yarip.setPrivateBrowsing = function(value)
{
    if (value == true || value == false) this.privateBrowsing = value;
}

yarip.setChangeType = function(type, force)
{
    if (!this.changeType || type > this.changeType || force) {
        this.changeType = type;
    }
}

yarip.createPage = function(location, address, privateBrowsing)
{
    if (!location) return null;

    if (!address) address = this.getAddress(location, this.mode);
    var page = new YaripPage(this.getId(), address);
    if (privateBrowsing) page.setTemporary(privateBrowsing);
    var aAddress = this.getAddressArray(address);
    if (this.exclusiveOnCreation || privateBrowsing) {
//        var exclusive = false;
//        for (var a in aAddress) {
//            if (this.map.get(a).contentWhitelist.getExclusive()) {
//              exclusive = true;
//              break;
//            }
//        }
//        if (!exclusive) {
        if (!aAddress[" _excl_ "]) { // not exclusive
            var sld = location.host.match(this.findSldRegExp);
            if (sld) page.setExclusive(location.protocol, sld[0], sld[2] !== undefined /* Is IP! */);
        }
    }
    if (this.templates.length > 0) {
        for (var i = 0; i < this.templates.length; i++) {
            page.merge(this.map.get(this.templates[i]));
        }
        for (var a in aAddress) {
            page.subtract(this.map.get(a));
        }
    }

    return page;
}

yarip.getAddress = function(location, mode)
{
//    if (!location) return null;
//    if (!/^https?:$/.test(location.protocol)) return null;
//    if (!location || !/^https?:$/.test(location.protocol)) return null;
    if (!location || (!/^https?:$/.test(location.protocol) && !/^https?$/.test(location.scheme))) return null;

    var mode = Number(mode);
    if (typeof mode != "number" || mode + "" == "NaN") mode = this.mode;

    var address = null;
    try
    {
        switch (mode)
        {
            case this.PAGE_MODE:
                if (location.protocol) address = location.protocol + "//" + location.host + location.pathname;
                else if (location.scheme) address = location.scheme + "://" + location.host + location.path;
                break;

            case this.FQDN_MODE:
                address = location.host;
                break;

            case this.SLD_MODE:
                var sld = location.host.match(this.findSldRegExp);
                if (sld) address = sld[0];
                break;

            case this.FIRSTDIR_MODE:
                if (location.protocol) {
                    var firstPath = location.pathname.match(/\/+[^/?]+/);
                    if (!firstPath) return null;
                    address = location.host + firstPath[0];
                } else if (location.scheme) {
                    var firstPath = location.path.match(/\/+[^/?]+/);
                    if (!firstPath) return null;
                    address = location.host + firstPath[0];
                }
                break;
        }
        if (address == "") return null;
    }
    catch (e){}
    return address;
}

yarip.getAddressArray = function(reduceDomain, follow)
{
    if (!reduceDomain) return null;

    var aAddress = {}
    var tmpDomain = null;

    while (reduceDomain != tmpDomain)
    {
        var reducePath = reduceDomain;
        var tmpPath = null;

        while (reducePath != tmpPath)
        {
            var page = this.map.get(reducePath);
            if (page) {
                aAddress[reducePath] = [true, true, true, true];
                if (page.contentWhitelist.getExclusive()) {
                    aAddress[" _excl_ "] = true; // flag exclusive
                }
            }
            tmpPath = reducePath;
            reducePath = reducePath.replace(/((:\d+)?\/|[^/]*)$/, "");
        }

        tmpDomain = reduceDomain;
        reduceDomain = reduceDomain.replace(/^([^:]+:\/\/|[\w-~!$&'()*+,;=:@]+\.)/, "");
    }

    if (follow) this.getExtensionAddressArray(aAddress, true);

    return aAddress;
}

yarip.getExtensionAddressArray = function(aAddress, append)
{
    if (!aAddress) return null;

    if (append) {
        for (var a in aAddress) {
            if (" _excl_ " === a && aAddress[a] === true) continue; // ignore
            this.getRecursiveAddressArray(a, aAddress, aAddress[a]);
        }
    } else {
        var aTmp = {}
        for (var a in aAddress) {
            if (" _excl_ " === a && aAddress[a] === true) continue; // ignore
            this.getRecursiveAddressArray(a, aTmp, aAddress[a]);
        }
        return aTmp;
    }

    return null;
}

yarip.getRecursiveAddressArray = function(parentAddress, aAddress, extMask)
{
    if (!extMask[1] && !extMask[2] && !extMask[3]) return;

    var extensions = this.map.get(parentAddress).extension.obj;

    for (var id in extensions)
    {
        var extPage = this.map.getById(id);
        if (extPage)
        {
            var extAddress = extPage.getName();
            var extension = extensions[id];
            if (extAddress in aAddress) // already added
            {
                if (extMask[1]) aAddress[extAddress][1] = aAddress[extAddress][1] || extension.getDoElements();
                if (extMask[2]) aAddress[extAddress][2] = aAddress[extAddress][2] || extension.getDoContents();
                if (extMask[3]) aAddress[extAddress][3] = aAddress[extAddress][3] || extension.getDoScripts();
            } else { // not added
                aAddress[extAddress] = [
                    false, // extended
                    extMask[1] && extension.getDoElements(),
                    extMask[2] && extension.getDoContents(),
                    extMask[3] && extension.getDoScripts()];
                if (extPage.extension.length > 0) {
                    this.getRecursiveAddressArray(extAddress, aAddress, aAddress[extAddress]);
                }
            }
        } else {
            this.map.get(parentAddress).extension.remove(id);
            this.setChangeType(this.ESSENTIAL_CHANGE);
        }
    }
}

yarip.resetOnAddress = function(obj)
{
    if (!obj) return false;

    var list = null;

    switch (obj.type)
    {
        case this.TYPE_WHITELIST:
            list = this.map.get(obj.address) ? this.map.get(obj.address).elementWhitelist : null;
            break;

        case this.TYPE_BLACKLIST:
            list = this.map.get(obj.address) ? this.map.get(obj.address).elementBlacklist : null;
            break;

        case this.TYPE_ATTRIBUTE:
            list = this.map.get(obj.address) ? this.map.get(obj.address).attributes : null;
            break;
    }

    switch (obj.type)
    {
        case this.TYPE_WHITELIST:
        case this.TYPE_BLACKLIST:
            var elements = this.getElementsByXPath(obj.document, obj.xValue);
            if (elements) for (var i = 0; i < elements.snapshotLength; i++)
            {
                var element = elements.snapshotItem(i);
                if (element.nodeType != 1) continue;

                switch (element.getAttribute("status"))
                {
                    case "whitelisted":
                    case "blacklisted":
//                    case "styled":
                        element.removeAttribute("status");
                }
            }
            break;
    }

    if (list)
    {
        list.remove(obj.key);
        this.setChangeType(this.ESSENTIAL_CHANGE);
    }
    delete this.undoObj[obj.type + "-" + obj.key];
    return true;
}

yarip.getIncrement = function(newIncrement, oldIncrement)
{
    return newIncrement > oldIncrement ? newIncrement : oldIncrement;
}

yarip.reloadPage = function(address, select)
{
    var noRefresh = this.checkPage(address);
    if (this.pageDialog && address) {
        this.pageDialog.reloadPage(address, !noRefresh, select);
        return true;
    } else {
        return false;
    }
}

yarip.checkPage = function(address)
{
    if (!address) return true;

    var page = this.map.get(address);
    if (!page) return true;
    if (page.isEmpty())
    {
        this.map.remove(page);
        return false;
    } else return true;
}

yarip.removeAllExceptWhitelisted = function(doc, pageAddress)
{
    this.whitelistXpath(doc, pageAddress, "/html/head/descendant-or-self::*[not(@status='blacklisted')]"); // whitelist head and descendants
    this.blacklistXpath(doc, pageAddress, "//*[not(@status='whitelisted')]"); // remove all except whitelisted
}

yarip.whitelistXpath = function(doc, address, xValue, isNew, xpath, doIncrement)
{
    if (!doc || !address || !xValue) return false;

    var elements = this.getElementsByXPath(doc, xValue);
    if (!elements) return false;

    var page = this.map.get(address);
    var elementWhitelist = page ? page.elementWhitelist : null;
    var found = false;
    var increment = this.INCREMENT_NOT_FOUND;

    for (var i = 0; i < elements.snapshotLength; i++)
    {
        var element = elements.snapshotItem(i);

        switch (element.nodeType)
        {
            // ELEMENT
            case 1:
                switch (element.getAttribute("status"))
                {
                    case "whitelisted":
                    case "blacklisted":
//                        increment = this.INCREMENT_IGNORED;
                        increment = this.getIncrement(this.INCREMENT_IGNORED, increment);
                        continue;
                }

                if (isNew) element.setAttribute("status", "whitelisted");
                else
                {
                    while (element && element.nodeType == 1 && element.getAttribute("status") != "whitelisted")
                    {
                        element.setAttribute("status", "whitelisted");
                        element = element.parentNode;
                    }
                }
                break;

            default:
                continue;
        }

        increment = this.INCREMENT_FOUND;
        found = true;
    }

    if (doIncrement && elementWhitelist && elementWhitelist.obj[xValue])
    {
        switch (increment)
        {
            case this.INCREMENT_FOUND:
                elementWhitelist.obj[xValue].incrementFound();
                break;

            case this.INCREMENT_NOT_FOUND:
                elementWhitelist.obj[xValue].incrementNotFound();
                break;

            case this.INCREMENT_IGNORED:
                elementWhitelist.obj[xValue].incrementIgnored();
                break;
        }

        this.setChangeType(this.MINOR_CHANGE);
    }

    if (isNew && found)
    {
//        if (!page) page = this.createPage(doc.location, address);
//        else page.setTemporary(false);
//        var tmpXPath = xpath ? xpath : new YaripWLElement(xValue);
//        tmpXPath.createStyle();
//        page.elementWhitelist.add(tmpXPath);
//        this.map.add(page);
//        this.setChangeType(this.ESSENTIAL_CHANGE);
        var tmpXPath = xpath ? xpath : new YaripWLElement(xValue);
        tmpXPath.createStyle();
        if (page) {
            page.setTemporary(false);
        }
        else {
            page = this.createPage(doc.location, address);
            this.map.add(page);
            this.map.sort();
        }
        page.elementWhitelist.add(tmpXPath);
        this.setChangeType(this.ESSENTIAL_CHANGE);

        this.reloadPage(address, false);

        var obj = {}
        obj.document = doc;
        obj.address = address;
        obj.xValue = xValue;
        obj.key = tmpXPath.getKey();
        obj.type = this.TYPE_WHITELIST;
        obj.text = xValue;
        this.undoObj[obj.type + "-" + obj.key] = obj;
    }

    return found;
}

yarip.blacklistXpath = function(doc, address, xValue, isNew, force, xpath, doIncrement)
{
    if (!doc || !address || !xValue) return false;

    var elements = this.getElementsByXPath(doc, xValue);
    if (!elements) return false;

    var page = this.map.get(address);
    var elementBlacklist = page ? page.elementBlacklist : null;
    var found = false;
    var increment = this.INCREMENT_NOT_FOUND;

    for (var i = 0; i < elements.snapshotLength; i++)
    {
        var element = elements.snapshotItem(i);

        switch (element.nodeType)
        {
            // ELEMENT
            case 1:
                switch (element.getAttribute("status"))
                {
                    case "whitelisted":
                    case "blacklisted":
                        if (!force)
                        {
//                            increment = this.INCREMENT_IGNORED;
                            increment = this.getIncrement(this.INCREMENT_IGNORED, increment);
                            continue;
                        }
                }

                switch (element.localName.toLowerCase())
                {
                    case "html":
                    case "body":
                    case "frameset":
                        continue;

                    case "head":
                        element.innerHTML =
                            "<style rel='stylesheet' type='text/css'>" +
                                "* {" +
                                    "margin: 0 !important;" +
                                    "padding: 0 !important;" +
                                "}" +
                                "div {" +
                                    "height: auto !important;" +
                                "}" +
                                "table {" +
                                    "border-spacing: 0 0 !important;" +
                                    "height: auto !important;" +
                                    "width: 0 !important;" +
                                "}" +
                            "</style>";
                        break;

                    case "script":
                        element.removeAttribute("src");
                        element.innerHTML = "";
                        break;

                    case "frame":
                        if (element.parentNode.localName == "FRAMESET") continue;

                        element.setAttribute("src", "about:blank");
                        element.setAttribute("status", "blacklisted");
                        break;

                    case "embed":
                    case "iframe":
                        element.setAttribute("src", "about:blank");
                        element.setAttribute("status", "blacklisted");
                        break;

                    default:
                        element.setAttribute("status", "blacklisted");
                        if (this.purgeInnerHTML && !isNew) element.innerHTML = "";
                }
                break;

            // ATTRIBUTE
            case 2:
                if (element.ownerElement) element.ownerElement.removeAttribute(element.name);
                break;

            // TEXT
            case 3:
                if (element.parentNode) element.parentNode.removeChild(element);
                break;

            default:
                continue;
        }

        increment = this.INCREMENT_FOUND;
        found = true;
    }

    if (doIncrement && elementBlacklist && elementBlacklist.obj[xValue])
    {
        switch (increment)
        {
            case this.INCREMENT_FOUND:
                elementBlacklist.obj[xValue].incrementFound();
                break;

            case this.INCREMENT_NOT_FOUND:
                elementBlacklist.obj[xValue].incrementNotFound();
                break;

            case this.INCREMENT_IGNORED:
                elementBlacklist.obj[xValue].incrementIgnored();
                break;
        }

        this.setChangeType(this.MINOR_CHANGE);
    }

    if (isNew && found)
    {
//        if (!page) page = this.createPage(doc.location, address);
//        else page.setTemporary(false);
//        var tmpXPath = xpath ? xpath : new YaripBLElement(xValue);
//        tmpXPath.createStyle();
//        page.elementBlacklist.add(tmpXPath);
//        this.map.add(page);
//        this.setChangeType(this.ESSENTIAL_CHANGE);
        var tmpXPath = xpath ? xpath : new YaripBLElement(xValue);
        tmpXPath.createStyle();
        if (page) {
            page.setTemporary(false);
        } else {
            page = this.createPage(doc.location, address);
            this.map.add(page);
            this.map.sort();
        }
        page.elementBlacklist.add(tmpXPath);
        this.setChangeType(this.ESSENTIAL_CHANGE);
        this.reloadPage(address, false);

        var obj = {}
        obj.document = doc;
        obj.address = address;
        obj.xValue = xValue;
        obj.key = tmpXPath.getKey();
        obj.type = this.TYPE_BLACKLIST;
        obj.text = xValue;
        this.undoObj[obj.type + "-" + obj.key] = obj;
    }
    return found;
}

yarip.whitelistContent = function(address, regExpValue, regexp, location)
{
    if (!address || !regExpValue) return false;

    var tmpRegExp = regexp ? regexp : new YaripWLContent(regExpValue);
    var page = this.map.get(address);
    if (page) {
        page.setTemporary(false);
    } else {
        page = this.createPage(location, address);
        this.map.add(page);
        this.map.sort();
    }
    page.contentWhitelist.add(tmpRegExp);
//    page.setTemporary(false);
//    this.map.add(page);
    this.setChangeType(this.ESSENTIAL_CHANGE);
    this.reloadPage(address, false);

    return true;
}

yarip.blacklistContent = function(address, regExpValue, regexp, location)
{
    if (!address || !regExpValue) return false;

    var tmpRegExp = regexp ? regexp : new YaripBLContent(regExpValue);
    var page = this.map.get(address);
    if (page) {
        page.setTemporary(false);
    } else {
        page = this.createPage(location, address);
        this.map.add(page);
        this.map.sort();
    }
    page.contentBlacklist.add(tmpRegExp);
//    page.setTemporary(false);
//    this.map.add(page);
    this.setChangeType(this.ESSENTIAL_CHANGE);
    this.reloadPage(address, false);

    return true;
}

yarip.extendPage = function(location, contentLocation)
{
    if (!location || !contentLocation) return false;

    var pageAddress = location.href;
    var aAddress = this.getAddressArray(pageAddress);
    pageAddress = null;
    for (var a in aAddress) {
        pageAddress = a;
        break;
    }

    if (!pageAddress) return false;

    var pageOwn = this.map.get(pageAddress);

    var aAddress = this.getAddressArray(contentLocation.spec);
    pageAddress = null;
    for (var a in aAddress) {
        pageAddress = a;
        break;
    }

    var page = null;
    if (pageAddress) {
        page = this.map.get(pageAddress);
    } else {
        location = {}
        location.href = contentLocation.spec;
        location.protocol = contentLocation.scheme + ":";
        location.host = contentLocation.host;
        location.pathname = contentLocation.path;
        pageAddress = this.getAddress(location, this.mode);
        page = this.createPage(location, pageAddress);
        this.map.add(page);
        this.map.sort();
    }

    pageOwn.extension.add(new YaripItem(page.getId(), false, true, false));
    pageOwn.extension.remove(pageOwn.getId());
    pageOwn.setTemporary(false);
    this.setChangeType(this.ESSENTIAL_CHANGE);
    this.reloadPage(pageAddress);
    this.reloadPage(pageOwn.getName(), false);

    return true;
}

yarip.setAttribute = function(doc, address, xValue, attrName, attrValue, isNew, xpath, doIncrement)
{
    if (!doc || !address || !xValue || !attrName || !attrValue) return false;

    var elements = this.getElementsByXPath(doc, xValue);
    if (!elements) return false;

    var page = this.map.get(address);
    var attributes = page ? page.attributes : null;
    var found = false;
    var increment = this.INCREMENT_NOT_FOUND;

    for (var i = 0; i < elements.snapshotLength; i++)
    {
        var element = elements.snapshotItem(i);
        switch (element.nodeType)
        {
            // ELEMENT
            case 1:
                switch (attrName)
                {
                    case "style":
                        var style = element.getAttribute(attrName);
                        element.setAttribute(attrName, style ? style + attrValue : attrValue);
    //                    element.setAttribute(attrName, "styled");
                        break;
                    default:
                        element.setAttribute(attrName, attrValue);
                }
                break;

            default:
                continue;
        }

        increment = this.INCREMENT_FOUND;
        found = true;
    }

    var key = xValue + attrName;
    if (doIncrement && attributes && attributes.obj[key])
    {
        switch (increment)
        {
            case this.INCREMENT_FOUND:
                attributes.obj[key].incrementFound();
                break;

            case this.INCREMENT_NOT_FOUND:
                attributes.obj[key].incrementNotFound();
                break;
        }

        this.setChangeType(this.MINOR_CHANGE);
    }

    if (isNew && found)
    {
//        if (!page) page = this.createPage(doc.location, address);
//        else page.setTemporary(false);
//        var tmpXPath = xpath ? xpath : new YaripAttribute(xValue, attrName, attrValue);
//        page.attributes.add(tmpXPath);
//        this.map.add(page);
//        this.setChangeType(this.ESSENTIAL_CHANGE);
        var tmpXPath = xpath ? xpath : new YaripAttribute(xValue, attrName, attrValue);
        if (page) {
            page.setTemporary(false);
        } else {
            page = this.createPage(doc.location, address);
            this.map.add(page);
            this.map.sort();
        }
        page.attributes.add(tmpXPath);
        this.setChangeType(this.ESSENTIAL_CHANGE);
        this.reloadPage(address, false);

        var obj = {}
        obj.document = doc;
        obj.address = address;
//        obj.xValue = xValue;
        obj.key = tmpXPath.getKey();
        obj.type = this.TYPE_ATTRIBUTE;
        obj.text = style + ":" + attrValue;
        this.undoObj[obj.type + "-" + obj.key] = obj;
    }

    return found;
}

yarip.calculateXPath = function(element, fullPath)
{
    if (!element || !element.localName) return null;

    if (element.nodeType == 2) return this.calculateXPath(element.ownerElement, fullPath) + "/@" + element.localName.toLowerCase(); // attribute

    var xValue = "";
    do
    {
        var attributes = "";
        if (element.getAttribute("id"))
        {
            // ID
            attributes += "[@id='" + element.getAttribute("id") + "']";
            if (!fullPath) return "//" + element.localName.toLowerCase() + attributes + xValue; // shortest xpath
        }
        else
        {
            // ATTRIBUTES
            if (element.getAttribute("name")) attributes += "@name='" + element.getAttribute("name") + "' and ";
            if (element.getAttribute("class")) attributes += "@class='" + element.getAttribute("class") + "' and ";
            if (attributes != "") attributes = "[" + attributes.substring(0, attributes.length - 5) + "]";

            // INDEX
            if (!/^(html|head|title|body|thead|tbody|tfoot|frameset)$/i.test(element.localName))
            {
                var index = 1;
                var sibling = element.previousSibling;
                while (sibling)
                {
//                    if (sibling.localName && element.localName && sibling.localName.toLowerCase() == element.localName.toLowerCase()) ++index;
                    if (sibling.localName == element.localName) ++index;
                    sibling = sibling.previousSibling;
                }

                var found = false;
                sibling = element.nextSibling;
                while (!found && sibling)
                {
//                    if (sibling.localName && element.localName && sibling.localName.toLowerCase() == element.localName.toLowerCase()) found = true;
                    if (sibling.localName == element.localName) found = true;
                    else sibling = sibling.nextSibling;
                }
                if (found || index > 1 || this.alwaysUseIndex) attributes = "[" + index + "]" + attributes;
            }
        }
        xValue = "/" + element.localName.toLowerCase() + attributes + xValue;

        element = element.parentNode;
    } while (element && element.nodeType == 1);

    return xValue != "" ? xValue : null;
}

yarip.getElementsByXPath = function(doc, xValue)
{
    var elements = null;
    try { elements = doc.evaluate(xValue, doc, null, Components.interfaces.nsIDOMXPathResult.UNORDERED_NODE_SNAPSHOT_TYPE, null); }
    catch (e)
    {
        var consoleService = Components.classes['@mozilla.org/consoleservice;1'].getService(Components.interfaces.nsIConsoleService);
        var scriptError = Components.classes['@mozilla.org/scripterror;1'].createInstance(Components.interfaces.nsIScriptError);
//        var message = doc.getElementById("yarip-overlay-stringbundle").getFormattedString("malformedXpath", [xValue]);
        var message = "[xpath=" + xValue + "] ";
//        var sourceName = "chrome://yarip/content/yarip.js";
        var sourceName = "";
        scriptError.init(message + e.message, sourceName, null, e.lineNumber, e.columnNumber, scriptError.errorFlag, "content javascript");
        consoleService.logMessage(scriptError);
    }
//    if (!elements || elements.snapshotLength == 0) return null;
    return elements;
}

yarip.generateRegExp = function(value)
{
    if (!value) return null;
//    return "^" + value.replace(/([/.*+?|()\[\]{}\\])/g, "\\$1") + "$";
    return "^" + value.replace(/([.*+?|()\[\]{}\\])/g, "\\$1") + "$";
}

yarip.checkXPath = function(xValue)
{
    if (!xValue) return false;
    try
    {
        var dp = Components.classes['@mozilla.org/xmlextras/domparser;1'].createInstance(Components.interfaces.nsIDOMParser);
        var doc = dp.parseFromString("<yarip></yarip>", "text/xml");
//        doc.evaluate(xValue, doc, null, Components.interfaces.nsIDOMXPathResult.ANY_TYPE, null);
        doc.evaluate(xValue, doc, null, Components.interfaces.nsIDOMXPathResult.UNORDERED_NODE_SNAPSHOT_TYPE, null);
    }
    catch (e) { return false; }
    return true;
}

yarip.checkRegExp = function(reValue)
{
    if (!reValue) return false;
    try { new RegExp(reValue).test(""); }
    catch (e) { return false; }
    return true;
}

yarip.getElements = function(doc, xValue)
{
    var elements = null;
    try { elements = doc.evaluate(xValue, doc, null, Components.interfaces.nsIDOMXPathResult.UNORDERED_NODE_SNAPSHOT_TYPE, null); } catch (e) {}
    return elements;
}

yarip.highlight = function(doc, xValue)
{
    if (!doc || !xValue) return false;
    var elements = this.getElements(doc, xValue);
    if (!elements) return false;
    if (elements.snapshotLength > 0) {
        for (var i = 0; i < elements.snapshotLength; i++) {
            var element = elements.snapshotItem(i);
            if (element && element.getAttribute) {
                var status = element.getAttribute("status");
                if (!status) {
                    element.setAttribute("status", "highlighted");
                } else if (!/\bhighlighted\b/.test(status)) {
                    element.setAttribute("status", status + " highlighted");
                }
            }
        }
    }
    return true;
}

//yarip.unHighlight = function(doc, xValue)
//{
//    if (!doc || !xValue) return;
//    var elements = this.getElements(doc, xValue);
//    if (elements && elements.snapshotLength > 0) {
//        for (var i = 0; i < elements.snapshotLength; i++) {
//            var element = elements.snapshotItem(i);
////            if (element.getAttribute && element.getAttribute("status") && /\bhighlighted\b/.test(element.getAttribute("status"))) {
//            if (element.getAttribute && /\bhighlighted\b/.test(element.getAttribute("status"))) {
//                element.setAttribute("status", element.getAttribute("status").replace(/\s*highlighted\b/, ""));
//            }
//        }
//    }
//}

yarip.unHighlight = function(doc, xValue)
{
    if (!doc || !xValue) return;
    var elements = this.getElements(doc, xValue);
    if (elements && elements.snapshotLength > 0) {
        for (var i = 0; i < elements.snapshotLength; i++) {
            var element = elements.snapshotItem(i);
            if (element && element.getAttribute) {
                var status = element.getAttribute("status");
                if (status && /\bhighlighted\b/.test(status)) {
                    var newStatus = status.replace(/\s*highlighted\b/, "");
                    if (newStatus == "") {
                        element.removeAttribute("status");
                    } else {
                        element.setAttribute("status", newStatus);
                    }
                }
            }

        }
    }
}

yarip.save = function()
{
    if (this.changeType == this.NO_CHANGE) return;

    // SAVE XPATHS
    var file = this.getFile(this.XPATH);
    this.map.sort(true);
    var data = this.map.generateXml();
    this.saveToFile(data, file);

    if (this.changeType == this.ESSENTIAL_CHANGE)
    {
        // SAVE STYLES
        file = this.getFile(this.STYLE);
        data = this.map.generateCSS();
        this.saveToFile(data, file);

        if (file)
        {
            var uri = "file://" + file.path;
            this.injectCSS(uri, false);
            this.injectCSS(uri, true);
        }
    }

    this.setChangeType(this.NO_CHANGE, true);
//    this.setValue(this.RELOAD, new Date().getTime(), this.STRING); // notify
}

yarip.getValue = function(preference, defaultValue, type)
{
    try
    {
        switch (type)
        {
            case this.BOOLEAN:
                return prefBranch.getBoolPref(preference);

            case this.INTEGER:
                return prefBranch.getIntPref(preference);

            case this.STRING:
                return prefBranch.getCharPref(preference);
        }
    }
    catch (e)
    {
        if (defaultValue != false && (typeof defaultValue != "number" || defaultValue + "" == "NaN") && !defaultValue) return null;
        this.setValue(preference, defaultValue, type);
    }

    return this.getValue(preference, null, type);
}

yarip.setValue = function(preference, value, type)
{
    try
    {
        switch (type)
        {
            case this.BOOLEAN:
                prefBranch.setBoolPref(preference, "" + value == "true");
                break;

            case this.INTEGER:
                prefBranch.setIntPref(preference, value);
                break;

            case this.STRING:
                prefBranch.setCharPref(preference, value);
                break;

            case this.RESET:
                prefBranch.clearUserPref(preference);
                break;
        }
    }
    catch (e){}
}

yarip.deleteBranch = function(branch)
{
    if (!branch) return;

    prefBranch.deleteBranch(branch);
}

yarip.injectCSS = function(uri, value)
{
    if (!uri) return;

    var ios = Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService);
    var uri = ios.newURI(uri, this.CHARSET, null); // uri, charset, baseuri
    var sss = Components.classes["@mozilla.org/content/style-sheet-service;1"].getService(Components.interfaces.nsIStyleSheetService);
    if (value) { if (!sss.sheetRegistered(uri, sss.USER_SHEET)) sss.loadAndRegisterSheet(uri, sss.USER_SHEET); }
    else { if (sss.sheetRegistered(uri, sss.USER_SHEET)) sss.unregisterSheet(uri, sss.USER_SHEET); }
}

yarip.getCurrentVersion = function()
{
    return Components.classes["@mozilla.org/extensions/manager;1"].getService(Components.interfaces.nsIExtensionManager).getItemForID("yarip@mozdev.org").version;
}

yarip.check = function(file)
{
    // FIRST RUN
    var firstrun = this.getValue(this.FIRSTRUN, true, this.BOOLEAN);
    if (firstrun)
    {
        this.setValue(this.FIRSTRUN, false, this.BOOLEAN);
    }

    // UPGRADE
    var previous = this.getValue(this.VERSION, "", this.STRING);
    var current = this.getCurrentVersion();
    if (previous != current && !firstrun)
    {
        this.setValue(this.VERSION, current, this.STRING);

        if (/^0\.1\.[2-7]$/.test(previous) && !/^0\.1\.[2-7]$/.test(current))
        {
            // CREATED STYLES FOR XPATHS
            for (var p in this.map.obj) for (var x in this.map.get(p).elementBlacklist.obj) this.map.get(p).elementBlacklist.obj[x].createStyle(true);
            this.setChangeType(this.ESSENTIAL_CHANGE);
        }

        if (/^0\.(1\.[2-9]|2\.1)$/.test(previous) && !/^0\.(1\.[2-9]|2\.1)$/.test(current))
        {
            // CHANGED NOFLICKER FEATURE
            this.setValue("content.notify.ontimer", null, this.RESET);
            this.setValue("content.notify.backoffcount", null, this.RESET);
            this.setValue("nglayout.initialpaint.delay", null, this.RESET);
        }

        if (/^0\.2(\.[1-3])?$/.test(previous) && !/^0\.2(\.[1-3])?$/.test(current))
        {
            Components.classes["@mozilla.org/satchel/form-history;1"].getService(Components.interfaces.nsIFormHistory2).removeEntriesForName("style");
        }

        this.deleteBranch("extensions.yarip.changeType"); // >0.1.7
        this.deleteBranch("extensions.yarip.flickerType"); // >0.1.7
        this.deleteBranch("extensions.yarip.loggingEnabled"); // >0.2.1
        this.deleteBranch("extensions.yarip.deep"); // >0.2.2
        this.deleteBranch("extensions.yarip.profile"); // >0.2.2
        this.deleteBranch("extensions.yarip.reload"); // >0.2.2
    }
}

yarip.getId = function()
{
    var id = new Date().getTime();
    while (this.map.getById(id) != undefined && this.map.getById(id) != null) id++;
    return "" + id;
}

yarip.addMonitorDialog = function(monitorDialog)
{
    monitorDialog.id = new Date().getTime();
//    while (this.monitorDialog["" + monitorDialog.id] != undefined && this.monitorDialog["" + monitorDialog.id] != null) monitorDialog.id++;
    while (this.monitorDialog["" + monitorDialog.id] != undefined) monitorDialog.id++;
    this.monitorDialog["" + monitorDialog.id] = monitorDialog;
}

yarip.removeMonitorDialog = function(monitorDialog)
{
    delete this.monitorDialog["" + monitorDialog.id];
    monitorDialog.id = -1;
}

yarip.load = function(file, imported)
{
    var doc = this.getDoc(file);
    if (!doc) {
        if (!imported) {
            this.setChangeType(this.ESSENTIAL_CHANGE);
            this.save();
        }
        return null;
    }
    else if (doc.documentElement.nodeName == "parsererror") return null;

    var pageName = null;
    var map = new YaripMap();

    var iYarip = doc.evaluate("./yarip", doc, null, Components.interfaces.nsIDOMXPathResult.ORDERED_NODE_ITERATOR_TYPE, null);
    var nYarip = iYarip.iterateNext();
    if (!nYarip) return null;

    var version = nYarip.getAttribute("version");
    if (!version)
    {
        var iPage = doc.evaluate("./yarip/page[child::whitelist or child::blacklist]", doc, null, Components.interfaces.nsIDOMXPathResult.ORDERED_NODE_ITERATOR_TYPE, null);
        var nPage = iPage.iterateNext();
        while (nPage)
        {
            pageName = nPage.getAttribute("name");
            var id = nPage.getAttribute("id");
            var page = map.obj[pageName] ? map.obj[pageName] : new YaripPage(id ? id : this.getId(), pageName);

            // XPATH WHITELIST
            var iWhitelist = doc.evaluate("./whitelist", nPage, null, Components.interfaces.nsIDOMXPathResult.ORDERED_NODE_ITERATOR_TYPE, null);
            var nWhitelist = iWhitelist.iterateNext();
            if (nWhitelist)
            {
                var iXPath = doc.evaluate("./xpath", nWhitelist, null, Components.interfaces.nsIDOMXPathResult.ORDERED_NODE_ITERATOR_TYPE, null);
                var nXPath = iXPath.iterateNext();
                while (nXPath)
                {
                    page.elementWhitelist.add(new YaripWLElement(
                        nXPath.getAttribute("value"),
                        nXPath.getAttribute("deep"),
                        null,
                        nXPath.getAttribute("found"),
                        nXPath.getAttribute("notFound"),
                        nXPath.getAttribute("whitelisted")));
                    nXPath = iXPath.iterateNext();
                }
            }

            // XPATH BLACKLIST
            var iBlacklist = doc.evaluate("./blacklist", nPage, null, Components.interfaces.nsIDOMXPathResult.ORDERED_NODE_ITERATOR_TYPE, null);
            var nBlacklist = iBlacklist.iterateNext();
            if (nBlacklist)
            {
                var iXPath = doc.evaluate("./xpath", nBlacklist, null, Components.interfaces.nsIDOMXPathResult.ORDERED_NODE_ITERATOR_TYPE, null);
                var nXPath = iXPath.iterateNext();
                while (nXPath)
                {
                    page.elementBlacklist.add(new YaripBLElement(
                        nXPath.getAttribute("value"),
                        nXPath.getAttribute("deep"),
                        nXPath.getAttribute("force") == "true",
                        nXPath.getAttribute("dependsOnWhitelist"),
                        nXPath.getAttribute("style"),
                        null,
                        nXPath.getAttribute("found"),
                        nXPath.getAttribute("notFound"),
                        nXPath.getAttribute("whitelisted")));
                    nXPath = iXPath.iterateNext();
                }
            }

            map.add(page);
            nPage = iPage.iterateNext();
        }
    }
    else if (/^0\.2(\.[1-3])?$/.test(version))
    {
        var iPage = doc.evaluate("./page", nYarip, null, Components.interfaces.nsIDOMXPathResult.ORDERED_NODE_ITERATOR_TYPE, null);
        var nPage = iPage.iterateNext();
        while (nPage)
        {
            pageName = nPage.getAttribute("name");
            var id = nPage.getAttribute("id");
            var page = map.obj[pageName] ? map.obj[pageName] : new YaripPage(id ? id : this.getId(), pageName);

            // ELEMENT BLACKLIST
            var iElementBlacklist = doc.evaluate("./elementBlacklist", nPage, null, Components.interfaces.nsIDOMXPathResult.ORDERED_NODE_ITERATOR_TYPE, null);
            var nElementBlacklist = iElementBlacklist.iterateNext();
            if (nElementBlacklist)
            {
                var iElement = doc.evaluate("./element", nElementBlacklist, null, Components.interfaces.nsIDOMXPathResult.ORDERED_NODE_ITERATOR_TYPE, null);
                var nElement = iElement.iterateNext();
                while (nElement)
                {
                    var element = new YaripBLElement(
                        nElement.getAttribute("xpath"),
                        nElement.getAttribute("deep"),
                        nElement.getAttribute("force") == "true",
                        nElement.getAttribute("dow"),
                        nElement.getAttribute("style"),
                        null,
                        nElement.getAttribute("found"),
                        nElement.getAttribute("notFound"),
                        nElement.getAttribute("ignored"));
                    if (imported) element.createStyle();
                    page.elementBlacklist.add(element);
                    nElement = iElement.iterateNext();
                }
            }

            // CONTENT BLACKLIST
            var iContentBlacklist = doc.evaluate("./contentBlacklist", nPage, null, Components.interfaces.nsIDOMXPathResult.ORDERED_NODE_ITERATOR_TYPE, null);
            var nContent = iContentBlacklist.iterateNext();
            if (nContent)
            {
                var iContent = doc.evaluate("./content", nContent, null, Components.interfaces.nsIDOMXPathResult.ORDERED_NODE_ITERATOR_TYPE, null);
                var nContent = iContent.iterateNext();
                while (nContent)
                {
                    page.contentBlacklist.add(new YaripBLContent(
                        nContent.getAttribute("regexp"),
                        nContent.getAttribute("deep"),
                        nContent.getAttribute("force") == "true",
                        null,
                        nContent.getAttribute("ignored")));
                    nContent = iContent.iterateNext();
                }
            }

            map.add(page);
            nPage = iPage.iterateNext();
        }

        var iPage = doc.evaluate("./page", nYarip, null, Components.interfaces.nsIDOMXPathResult.ORDERED_NODE_ITERATOR_TYPE, null);
        var nPage = iPage.iterateNext();
        while (nPage)
        {
            pageName = nPage.getAttribute("name");
            var id = nPage.getAttribute("id");
            var page = map.obj[pageName] ? map.obj[pageName] : new YaripPage(id ? id : this.getId(), pageName);

            // ELEMENT WHITELIST
            var iElementWhitelist = doc.evaluate("./elementWhitelist", nPage, null, Components.interfaces.nsIDOMXPathResult.ORDERED_NODE_ITERATOR_TYPE, null);
            var nElementWhitelist = iElementWhitelist.iterateNext();
            if (nElementWhitelist)
            {
//                page.elementWhitelist.setExclusive(nElementWhitelist.getAttribute("exclusive"));
                var iElement = doc.evaluate("./element", nElementWhitelist, null, Components.interfaces.nsIDOMXPathResult.ORDERED_NODE_ITERATOR_TYPE, null);
                var nElement = iElement.iterateNext();
                while (nElement)
                {
                    page.elementWhitelist.add(new YaripWLElement(
                        nElement.getAttribute("xpath"),
                        nElement.getAttribute("deep"),
                        null,
                        nElement.getAttribute("found"),
                        nElement.getAttribute("notFound"),
                        nElement.getAttribute("ignored")));
                    nElement = iElement.iterateNext();
                }
            }

            // CONTENT WHITELIST
            var iContentWhitelist = doc.evaluate("./contentWhitelist", nPage, null, Components.interfaces.nsIDOMXPathResult.ORDERED_NODE_ITERATOR_TYPE, null);
            var nContent = iContentWhitelist.iterateNext();
            if (nContent)
            {
                page.contentWhitelist.setExclusive(nContent.getAttribute("exclusive"));
                var iContent = doc.evaluate("./content", nContent, null, Components.interfaces.nsIDOMXPathResult.ORDERED_NODE_ITERATOR_TYPE, null);
                var nContent = iContent.iterateNext();
                while (nContent)
                {
                    page.contentWhitelist.add(new YaripWLContent(
                        nContent.getAttribute("regexp"),
                        nContent.getAttribute("deep")));
                    nContent = iContent.iterateNext();
                }
            }

            // STYLES
            var iStyles = doc.evaluate("./styles", nPage, null, Components.interfaces.nsIDOMXPathResult.ORDERED_NODE_ITERATOR_TYPE, null);
            var nStyles = iStyles.iterateNext();
            if (nStyles)
            {
                var iElement = doc.evaluate("./element", nStyles, null, Components.interfaces.nsIDOMXPathResult.ORDERED_NODE_ITERATOR_TYPE, null);
                var nElement = iElement.iterateNext();
                while (nElement)
                {
                    page.attributes.add(new YaripAttribute(
                        nElement.getAttribute("xpath"),
                        "style",
                        nElement.getAttribute("style"),
                        nElement.getAttribute("deep"),
                        null,
                        nElement.getAttribute("found"),
                        nElement.getAttribute("notFound")));
                    nElement = iElement.iterateNext();
                }
            }

            // TODO Remove this, since it was a temporary change!
            // ATTRIBUTES
            var iAttributes = doc.evaluate("./attributes", nPage, null, Components.interfaces.nsIDOMXPathResult.ORDERED_NODE_ITERATOR_TYPE, null);
            var nAttributes = iAttributes.iterateNext();
            if (nAttributes)
            {
                var iElement = doc.evaluate("./element", nAttributes, null, Components.interfaces.nsIDOMXPathResult.ORDERED_NODE_ITERATOR_TYPE, null);
                var nElement = iElement.iterateNext();
                while (nElement)
                {
                    page.attributes.add(new YaripAttribute(
                        nElement.getAttribute("xpath"),
                        nElement.getAttribute("name"),
                        nElement.getAttribute("value"),
                        nElement.getAttribute("deep"),
                        nElement.getAttribute("lastFound"),
                        nElement.getAttribute("found"),
                        nElement.getAttribute("notFound")));
                    nElement = iElement.iterateNext();
                }
            }

            // EXTENSION
            var iExtension = doc.evaluate("./extension", nPage, null, Components.interfaces.nsIDOMXPathResult.ORDERED_NODE_ITERATOR_TYPE, null);
            var nExtension = iExtension.iterateNext();
            if (nExtension)
            {
                var iItem = doc.evaluate("./item", nExtension, null, Components.interfaces.nsIDOMXPathResult.ORDERED_NODE_ITERATOR_TYPE, null);
                var nItem = iItem.iterateNext();
                while (nItem)
                {
                    var value = nItem.getAttribute("value");
                    if (value != page.getId()) {
                        page.extension.add(new YaripItem(
                            nItem.getAttribute("value"),
                            nItem.getAttribute("doElements"),
                            nItem.getAttribute("doContents"),
                            nItem.getAttribute("doScripts")));
                    }
                    nItem = iItem.iterateNext();
                }
                page.extension.remove(page.getId()); // remove self
            }

            map.add(page);
            nPage = iPage.iterateNext();
        }
    }
    else
    {
        var iPage = doc.evaluate("./page", nYarip, null, Components.interfaces.nsIDOMXPathResult.ORDERED_NODE_ITERATOR_TYPE, null);
        var nPage = iPage.iterateNext();
        while (nPage)
        {
            pageName = nPage.getAttribute("name");
            var id = nPage.getAttribute("id");
            var page = map.obj[pageName] ? map.obj[pageName] : new YaripPage(id ? id : this.getId(), pageName);

            // ELEMENT WHITELIST
            var iElementWhitelist = doc.evaluate("./elementWhitelist", nPage, null, Components.interfaces.nsIDOMXPathResult.ORDERED_NODE_ITERATOR_TYPE, null);
            var nElementWhitelist = iElementWhitelist.iterateNext();
            if (nElementWhitelist)
            {
                var iElement = doc.evaluate("./element", nElementWhitelist, null, Components.interfaces.nsIDOMXPathResult.ORDERED_NODE_ITERATOR_TYPE, null);
                var nElement = iElement.iterateNext();
                while (nElement)
                {
                    var iXPath = doc.evaluate("./xpath", nElement, null, Components.interfaces.nsIDOMXPathResult.ORDERED_NODE_ITERATOR_TYPE, null);
                    var nXPath = iXPath.iterateNext();
                    if (nXPath) {
                        page.elementWhitelist.add(new YaripWLElement(
                            nXPath.textContent,
                            nElement.getAttribute("deep"),
                            nElement.getAttribute("lastFound"),
                            nElement.getAttribute("found"),
                            nElement.getAttribute("notFound"),
                            nElement.getAttribute("ignored")));
                    }
                    nElement = iElement.iterateNext();
                }
            }

            // ELEMENT BLACKLIST
            var iElementBlacklist = doc.evaluate("./elementBlacklist", nPage, null, Components.interfaces.nsIDOMXPathResult.ORDERED_NODE_ITERATOR_TYPE, null);
            var nElementBlacklist = iElementBlacklist.iterateNext();
            if (nElementBlacklist)
            {
                var iElement = doc.evaluate("./element", nElementBlacklist, null, Components.interfaces.nsIDOMXPathResult.ORDERED_NODE_ITERATOR_TYPE, null);
                var nElement = iElement.iterateNext();
                while (nElement)
                {
                    var iXPath = doc.evaluate("./xpath", nElement, null, Components.interfaces.nsIDOMXPathResult.ORDERED_NODE_ITERATOR_TYPE, null);
                    var nXPath = iXPath.iterateNext();
                    var iStyle = doc.evaluate("./style", nElement, null, Components.interfaces.nsIDOMXPathResult.ORDERED_NODE_ITERATOR_TYPE, null);
                    var nStyle = iStyle.iterateNext();
                    if (nXPath) {
                        var element = new YaripBLElement(
                            nXPath.textContent,
                            nElement.getAttribute("deep"),
                            nElement.getAttribute("force"),
                            nElement.getAttribute("dow"),
                            nStyle ? nStyle.textContent : null,
                            nElement.getAttribute("lastFound"),
                            nElement.getAttribute("found"),
                            nElement.getAttribute("notFound"),
                            nElement.getAttribute("ignored"));
                        if (imported) element.createStyle();
                        page.elementBlacklist.add(element);
                    }
                    nElement = iElement.iterateNext();
                }
            }

            // CONTENT WHITELIST
            var iContentWhitelist = doc.evaluate("./contentWhitelist", nPage, null, Components.interfaces.nsIDOMXPathResult.ORDERED_NODE_ITERATOR_TYPE, null);
            var nContent = iContentWhitelist.iterateNext();
            if (nContent)
            {
                page.contentWhitelist.setExclusive(nContent.getAttribute("exclusive"));
                var iContent = doc.evaluate("./content", nContent, null, Components.interfaces.nsIDOMXPathResult.ORDERED_NODE_ITERATOR_TYPE, null);
                var nContent = iContent.iterateNext();
                while (nContent)
                {
                    var iRegexp = doc.evaluate("./regexp", nContent, null, Components.interfaces.nsIDOMXPathResult.ORDERED_NODE_ITERATOR_TYPE, null);
                    var nRegexp = iRegexp.iterateNext();
                    if (nRegexp) {
                        page.contentWhitelist.add(new YaripWLContent(
                            nRegexp.textContent,
                            nContent.getAttribute("deep"),
                            nContent.getAttribute("lastFound")));
                    }
                    nContent = iContent.iterateNext();
                }
            }

            // CONTENT BLACKLIST
            var iContentBlacklist = doc.evaluate("./contentBlacklist", nPage, null, Components.interfaces.nsIDOMXPathResult.ORDERED_NODE_ITERATOR_TYPE, null);
            var nContent = iContentBlacklist.iterateNext();
            if (nContent)
            {
                var iContent = doc.evaluate("./content", nContent, null, Components.interfaces.nsIDOMXPathResult.ORDERED_NODE_ITERATOR_TYPE, null);
                var nContent = iContent.iterateNext();
                while (nContent)
                {
                    var iRegexp = doc.evaluate("./regexp", nContent, null, Components.interfaces.nsIDOMXPathResult.ORDERED_NODE_ITERATOR_TYPE, null);
                    var nRegexp = iRegexp.iterateNext();
                    if (nRegexp) {
                        page.contentBlacklist.add(new YaripBLContent(
                            nRegexp.textContent,
                            nContent.getAttribute("deep"),
                            nContent.getAttribute("force"),
                            nContent.getAttribute("lastFound"),
                            nContent.getAttribute("ignored")));
                    }
                    nContent = iContent.iterateNext();
                }
            }

            // ATTRIBUTES
            var iAttributes = doc.evaluate("./attributes", nPage, null, Components.interfaces.nsIDOMXPathResult.ORDERED_NODE_ITERATOR_TYPE, null);
            var nAttributes = iAttributes.iterateNext();
            if (nAttributes)
            {
                var iElement = doc.evaluate("./element", nAttributes, null, Components.interfaces.nsIDOMXPathResult.ORDERED_NODE_ITERATOR_TYPE, null);
                var nElement = iElement.iterateNext();
                while (nElement)
                {
                    var iXPath = doc.evaluate("./xpath", nElement, null, Components.interfaces.nsIDOMXPathResult.ORDERED_NODE_ITERATOR_TYPE, null);
                    var nXPath = iXPath.iterateNext();
                    var iName = doc.evaluate("./name", nElement, null, Components.interfaces.nsIDOMXPathResult.ORDERED_NODE_ITERATOR_TYPE, null);
                    var nName = iName.iterateNext();
                    var iValue = doc.evaluate("./value", nElement, null, Components.interfaces.nsIDOMXPathResult.ORDERED_NODE_ITERATOR_TYPE, null);
                    var nValue = iValue.iterateNext();
                    if (nXPath && nName && nValue) {
                        page.attributes.add(new YaripAttribute(
                            nXPath.textContent,
                            nName.textContent,
                            nValue.textContent,
                            nElement.getAttribute("deep"),
                            nElement.getAttribute("lastFound"),
                            nElement.getAttribute("found"),
                            nElement.getAttribute("notFound")));
                    }
                    nElement = iElement.iterateNext();
                }
            }

            // EXTENSION
            var iExtension = doc.evaluate("./extension", nPage, null, Components.interfaces.nsIDOMXPathResult.ORDERED_NODE_ITERATOR_TYPE, null);
            var nExtension = iExtension.iterateNext();
            if (nExtension)
            {
                var iPageRef = doc.evaluate("./pageReference", nExtension, null, Components.interfaces.nsIDOMXPathResult.ORDERED_NODE_ITERATOR_TYPE, null);
                var nPageRef = iPageRef.iterateNext();
                while (nPageRef)
                {
                    var pageId = nPageRef.textContent;
                    if (pageId && pageId != page.getId()) {
                        page.extension.add(new YaripItem(
                            pageId,
                            nPageRef.getAttribute("doElements"),
                            nPageRef.getAttribute("doContents"),
                            nPageRef.getAttribute("doScripts")));
                    }
                    nPageRef = iPageRef.iterateNext();
                }
                page.extension.remove(page.getId()); // remove self
            }

            // SCRIPT
            var iScript = doc.evaluate("./script", nPage, null, Components.interfaces.nsIDOMXPathResult.ORDERED_NODE_ITERATOR_TYPE, null);
            var nScript = iScript.iterateNext();
            if (nScript) {
                page.script.setLastChanged(nScript.getAttribute("lastChanged"));
                page.script.set(null, null, nScript.textContent);
            }

            map.add(page);
            nPage = iPage.iterateNext();
        }
    }

    if (imported) {
        this.map.merge(map);
    } else {
        this.map = map;
    }

    return pageName;
}

yarip.getDoc = function(file)
{
    if (!file) return null;

    var data = this.getData(file);
    if (!data || data == "") return null;

    var dp = Components.classes['@mozilla.org/xmlextras/domparser;1'].createInstance(Components.interfaces.nsIDOMParser);
    return dp.parseFromString(data, "text/xml");
}

yarip.getData = function(file)
{
    if (!file) return "";

    var fis = null;
    var sis = null;
    var data = null;
    try {
        fis = Components.classes['@mozilla.org/network/file-input-stream;1'].createInstance(Components.interfaces.nsIFileInputStream);
        fis.init(file, 0x01, 00004, null);
        sis = Components.classes['@mozilla.org/scriptableinputstream;1'].createInstance(Components.interfaces.nsIScriptableInputStream);
        sis.init(fis);
        data = sis.read(sis.available());
        var suc = Components.classes["@mozilla.org/intl/scriptableunicodeconverter"].createInstance(Components.interfaces.nsIScriptableUnicodeConverter);
        suc.charset = this.CHARSET;
        var data = suc.ConvertToUnicode(data);
    } catch (e) {
    } finally {
        if (sis != null) try { sis.close(); } catch (e) {}
        if (fis != null) try { fis.close(); } catch (e) {}
    }
    return data ? data : "";
}

yarip.getFile = function(path)
{
    if (!path) return null;
    var file = Components.classes["@mozilla.org/file/directory_service;1"].getService(Components.interfaces.nsIProperties).get("ProfD", Components.interfaces.nsIFile);
    var aPath = path.split(/\/|\\/);
    for (var i = 0; i < aPath.length; i++) file.append(aPath[i]);
    if (!file.exists()) file.create(Components.interfaces.nsIFile.NORMAL_FILE_TYPE, 0600);
    if (file.isFile()) return file;
    else return null;
}

yarip.saveToFile = function(data, file)
{
    if (data != "" && !data || !file) return;

    var fos = null;
    var os = null;
    try {
        fos = Components.classes["@mozilla.org/network/file-output-stream;1"].createInstance(Components.interfaces.nsIFileOutputStream);
        fos.init(file, 0x02 | 0x08 | 0x20, 0600, 0);
//        fos.write(data, data.length);

        os = Components.classes["@mozilla.org/intl/converter-output-stream;1"].createInstance(Components.interfaces.nsIConverterOutputStream);
        os.init(fos, this.CHARSET, 0, 0x0000);
        os.writeString(data);
    } catch (e) {
    } finally {
        if (fos != null) try { fos.close(); } catch (e) {}
        if (os != null) try { os.close(); } catch (e) {}
    }
}

/*
 * nsIContentPolicy
 */

yarip.shouldLoad = function(contentType, contentLocation, requestOrigin, context, mimeTypeGuess, extra)
{
    if (!this.enabled || !this.schemesRegExp /*|| contentType == TYPE_DOCUMENT*/
        || !contentLocation || !requestOrigin || !context
        || !this.schemesRegExp.test(contentLocation.scheme)) return ACCEPT;
//        || !this.schemesRegExp.test(requestOrigin.scheme)) return ACCEPT;

    var defaultView = null;
    if (context.nodeType != DOCUMENT_NODE) context = context.ownerDocument;
    if (context && context.nodeType == DOCUMENT_NODE) defaultView = context.defaultView;
    if (!defaultView) return ACCEPT;

    var doc = defaultView.document;
    if (!doc) return ACCEPT;

    var pageAddress = this.getAddress(doc.location, this.PAGE_MODE);
    if (!pageAddress) return ACCEPT;

    // TODO Reuse aAddress for all content on the same page!?
    var aAddress = this.getAddressArray(pageAddress, true);
    var hasAddress = false;
    for (var a in aAddress) { hasAddress = true; break; }
    if (!hasAddress && this.privateBrowsing) {
        var page = this.createPage(doc.location, null, true);
//        if (page) {
            this.map.add(page);
            this.map.sort();
            this.reloadPage(page.getName(), false);
            aAddress = this.getAddressArray(pageAddress, true);
//        }
    }

    var date = new Date();
    switch (this.shouldBlacklistContent(pageAddress, aAddress, contentLocation.spec, defaultView))
    {
        case 0:
            for (var id in this.monitorDialog) this.monitorDialog[id].logContent(0, doc.location, this.mode, contentLocation, date, mimeTypeGuess);
            return ACCEPT;
        case 1:
            for (var id in this.monitorDialog) this.monitorDialog[id].logContent(1, doc.location, this.mode, contentLocation, date, mimeTypeGuess);
            return ACCEPT;
        case 2:
            for (var id in this.monitorDialog) this.monitorDialog[id].logContent(2, doc.location, this.mode, contentLocation, date, mimeTypeGuess);
            return REJECT_SERVER;
        default:
            for (var id in this.monitorDialog) this.monitorDialog[id].logContent(0, doc.location, this.mode, contentLocation, date, mimeTypeGuess);
            return ACCEPT;
    }
}

yarip.shouldProcess = function(contentType, contentLocation, requestOrigin, context, mimeType, extra)
{
    return ACCEPT;
}

yarip.shouldBlacklistContent = function(pageAddress, aAddress, url, defaultView)
{
//    if (!aAddress || !url) return false;

    var whitelisted = false;
    var exclusive = false;
    var result = 0;

    for (var a in aAddress)
    {
        if (whitelisted) break;
        if (!aAddress[a][2]) continue;
        var contentWhitelist = this.map.get(a).contentWhitelist;
        exclusive = exclusive || contentWhitelist.getExclusive();
//        if (whitelisted) break;
        if (contentWhitelist.length > 0)
        {
            var list = contentWhitelist.obj;
            var equalAddresses = this.equalAddresses(pageAddress, a);
            for (var k in list)
            {
                var content = list[k];

                if (!content.getDeep() && !equalAddresses) continue;
                if (content.getRegExpObj().test(url))
                {
                    if (aAddress[a][0]) { // is self
                        content.incrementLastFound();
                        this.setChangeType(this.MINOR_CHANGE);
                    }
                    defaultView.yaripStatus = "found";
                    whitelisted = true;
                    result = 1;
                    break;
                }
            }
        }
    }

    for (var a in aAddress)
    {
        if (!aAddress[a][2]) continue;
        var contentBlacklist = this.map.get(a).contentBlacklist;
        if (contentBlacklist.length > 0)
        {
            var list = contentBlacklist.obj;
            var equalAddresses = this.equalAddresses(pageAddress, a);
            for (var k in list)
            {
                var content = list[k];
                if (!content.getDeep() && !equalAddresses) continue;
                if (content.getRegExpObj().test(url))
                {
                    if (whitelisted && !content.getForce())
                    {
                        if (aAddress[a][0]) {
                            content.incrementIgnored();
                            this.setChangeType(this.MINOR_CHANGE);
                        }
                        continue;
                    }

                    if (aAddress[a][0]) { // is self
                        content.incrementLastFound();
                        this.setChangeType(this.MINOR_CHANGE);
                    }
                    defaultView.yaripStatus = "found";
                    return 2;
                }
            }
        }
    }

    return !whitelisted && exclusive ? 2 : result;
}

yarip.equalAddresses = function(pageAddress, address)
{
    if (pageAddress.indexOf(address) == -1) return true; // ignore deep (extension)
    pageAddress = pageAddress.replace(/\/+$/, "");
    address = address.replace(/\/+$/, "");
    if (this.noSchemeRegExp.test(address)) pageAddress = pageAddress.replace(/^[^:/?#]+:\/{2}/, ""); // remove scheme
    return pageAddress == address;
}

/*
 * nsIObserverService
 */

const CACHE_SERVICE = Components.classes["@mozilla.org/network/cache-service;1"].getService(Components.interfaces.nsICacheService);

yarip.observe = function(aSubject, aTopic, aData)
{
    switch (aTopic)
    {
        case "http-on-examine-response":
        case "http-on-examine-cached-response":
            var httpChannel = aSubject.QueryInterface(Components.interfaces.nsIHttpChannel);
            var uri = aSubject.URI.asciiSpec;
            var contentType = aSubject.contentType;
            var statusCode = "";
            try{ statusCode = aSubject.responseStatus; }catch(e){}
            for (var id in this.monitorDialog) this.monitorDialog[id].updateContentType(uri, contentType, statusCode);
            break;
    }
}

yarip.QueryInterface = function(aIID)
{
    if (Components.interfaces.nsIContentPolicy.equals(aIID)
//        || Components.interfaces.nsIObserverService.equals(aIID)
        || Components.interfaces.nsISupports.equals(aIID))
    {
        return this;
    }
    throw Components.results.NS_ERROR_NO_INTERFACE;
}

yarip.wrappedJSObject = yarip;

var factory = {
    createInstance: function (aOuter, aIID)
    {
        if (aOuter != null) throw Components.results.NS_ERROR_NO_AGGREGATION;

        var ssl = Components.classes["@mozilla.org/moz/jssubscript-loader;1"].getService(Components.interfaces.mozIJSSubScriptLoader);
        ssl.loadSubScript("chrome://yarip/content/map.js");
        ssl.loadSubScript("chrome://yarip/content/page.js");
        ssl.loadSubScript("chrome://yarip/content/list.js");
        ssl.loadSubScript("chrome://yarip/content/xpath.js");

        yarip.map = new YaripMap();

        // TODO Remove the log-file from the file-system!
//        yarip.logFile = yarip.getFile(yarip.LOG);
//        yarip.saveToFile("", yarip.logFile); // purge

//        var file = yarip.getFile(yarip.XPATH);
//        yarip.load(file);
//        yarip.check(file);

        yarip.toggleEnabled(yarip.getValue(yarip.ENABLED, true, yarip.BOOLEAN));
//        yarip.setMode(yarip.FQDN_MODE);
        yarip.setMode(yarip.getValue(yarip.MODE, yarip.FQDN_MODE, yarip.INTEGER));
        yarip.setAlwaysUseIndex(yarip.getValue(yarip.INDEX, false, yarip.BOOLEAN));
        yarip.setElementsInContext(yarip.getValue(yarip.ELEMENTS, 4, yarip.INTEGER));
        yarip.setPurgeInnerHTML(yarip.getValue(yarip.PURGE, false, yarip.BOOLEAN));
        yarip.setExclusiveOnCreation(yarip.getValue(yarip.EXCLUSIVE, false, yarip.BOOLEAN));
        yarip.setTemplates(yarip.getValue(yarip.TEMPLATES, "", yarip.STRING));
        yarip.setSchemes(yarip.getValue(yarip.SCHEMES, "^https?$", yarip.STRING));
        yarip.setPrivateBrowsing(yarip.getValue(yarip.PRIVATE, "false", yarip.BOOLEAN));

        var os = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);
        os.addObserver(yarip, "http-on-examine-response", false);
        os.addObserver(yarip, "http-on-examine-cached-response", false);

        return yarip.QueryInterface(aIID);
    },

    QueryInterface: function (aIID)
    {
        if (Components.interfaces.nsIFactory.equals(aIID)
            || Components.interfaces.nsISupports.equals(aIID))
        {
            return this;
        }
        throw Components.results.NS_ERROR_NO_INTERFACE;
    }
}

var module = {
    registerSelf: function(aCompMgr, aFileSpec, aLocation, aType)
    {
        var cr = aCompMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);
        cr.registerFactoryLocation(CLASS_ID, CLASS_NAME, CONTRACT_ID, aFileSpec, aLocation, aType);

        var cm = Components.classes["@mozilla.org/categorymanager;1"].getService(Components.interfaces.nsICategoryManager);
        cm.deleteCategoryEntry("content-policy", CONTRACT_ID, true);
        cm.addCategoryEntry("content-policy", CONTRACT_ID, CONTRACT_ID, true, true);
    },

    unregisterSelf: function(aCompMgr, aLocation, aType)
    {
        var cr = aCompMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);
        cr.unregisterFactoryLocation(CLASS_ID, aLocation);

        var cm = Components.classes["@mozilla.org/categorymanager;1"].getService(Components.interfaces.nsICategoryManager);
        cm.deleteCategoryEntry("content-policy", CONTRACT_ID, true);
    },

    getClassObject: function(aCompMgr, aCID, aIID)
    {
        if (!Components.interfaces.nsIFactory.equals(aIID)) throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
        if (CLASS_ID.equals(aCID)) return factory;
        throw Components.results.NS_ERROR_NO_INTERFACE;
    },

    canUnload: function(aCompMgr) { return true; }
}

function NSGetModule(aCompMgr, aFileSpec) { return module; }
