
/*
Copyright 2007-2009 Kim A. Brandt <kimabrandt@gmx.de>

This file is part of yarip.

Yarip is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Yarip is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with yarip; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

function YaripHandler(doc, stopFunc)
{
    this.doc = doc ? doc : null;
    this.stopFunc = stopFunc ? stopFunc : null;

    this.element = null;
    this.timeout = null;
    this.outline = null;
    this.xValue = null;
    this.address = null;
    this.handlers = [];
    this.active = false;
    this.changesMade = false;
    this.hasTmpWhitelist = false;
    this.hasTmpBlacklist = false;
    this.selectStack = null;

    this.start = function(doc)
    {
        if (doc) this.doc = doc;

        this.address = yarip.getAddress(this.doc.location);
        this.active = false;
        this.changesMade = false;
        this.hasTmpWhitelist = false;
        this.hasTmpBlacklist = false;
//        this.selectStack = new Array();

        if (this.doc && this.doc.nodeName == "#document" && this.address)
        {
            this.doc.addEventListener("mouseover", this, false);
            this.doc.addEventListener("mouseout", this, false);
            this.doc.addEventListener("mousedown", this, false);
            this.doc.addEventListener("keypress", this, false);
            this.active = true;
        }

        var frames = this.doc.defaultView.frames;
        if (frames) for (var i = 0; i < frames.length; i++) this.addHandler(new YaripHandler(frames[i].document, function() { this.stopFunc(); }));

        if (this.handlers.length > 0) for (var i = 0; i < this.handlers.length; i++) if (this.handlers[i].start(null)) this.active = true;

        if (this.doc.defaultView) this.doc.defaultView.focus();

        return this.active;
    }

    this.stop = function()
    {
        if (!this.active || !this.doc) return false;

        this.doc.removeEventListener("mouseover", this, false);
        this.doc.removeEventListener("mouseout", this, false);
        this.doc.removeEventListener("mousedown", this, false);
        this.doc.removeEventListener("keypress", this, false);

        if (this.handlers.length > 0) for (var i = 0; i < this.handlers.length; i++) if (this.handlers[i].stop()) this.changesMade = true;
        this.handlers = [];

        if (this.hasTmpWhitelist && !this.hasTmpBlacklist) yarip.removeAllExceptWhitelisted(this.doc, this.address);

        this.reset();
        this.active = false;

        return this.changesMade;
    }

    this.addHandler = function(handler)
    {
        if (!handler) return;

        this.handlers[this.handlers.length] = handler;
    }

    this.handleEvent = function(event)
    {
        if (!event.target.localName || /^(frameset|i?frame)$/i.test(event.target.localName)) return; // ignore elements

        switch (event.type)
        {
            case "mouseover":
                if (/^(html|body)$/i.test(event.target.localName)) return;
                if (this.element) this.reset();
                this.element = event.target;
                if (!this.element.href) this.switchWithAnchorAncestor();
                this.selectStack = [];
                this.timeout = setTimeout(this.select, 100, this);
                event.stopPropagation();
                break;

            case "mouseout":
                if (!this.element) break;
                this.reset();
                event.stopPropagation();
                break;

            case "mousedown":
                switch (event.button)
                {
                    case 0: // left (blacklist)
                        this.doBlacklisting();
                        break;

                    case 1: // middle (blacklist temporarily)
                        this.doBlacklisting(true);
                        break;
                }
                event.stopPropagation();
                break;

            case "keypress":
                switch (event.keyCode)
                {
                    case 27: // escape
                        this.stopFunc();
                        event.stopPropagation();
                        break;

                    case 0: // character
                        switch (event.charCode)
                        {
                            case 119: // `w' (whitelist)
                                this.doWhitelisting();
                                event.stopPropagation();
                                break;

                            case 107: // `k' (keep, whitelist temporarily)
                                this.doWhitelisting(true);
                                event.stopPropagation();
                                break;

                            case 98: // `b' (blacklist)
                                this.doBlacklisting();
                                event.stopPropagation();
                                break;

                            case 99: // `c' (child select)
                                if (!this.element) return;
                                var child = this.selectStack.pop();
                                if (!child) return;
                                this.reset();
                                this.element = child;
                                this.select(this);
                                event.stopPropagation();
                                break;

                            case 112: // `p' (parent select)
                                if (!this.element) return;
                                var parent = this.element.parentNode;
                                if (!parent || /^(html|body)$/i.test(parent.localName)) return;
                                this.selectStack.push(this.element);
                                this.reset();
                                this.element = parent;
                                if (!this.element.href) this.switchWithAnchorAncestor();
                                this.select(this);
                                event.stopPropagation();
                                break;

                            case 114: // `r' (remove, blacklist temporarily)
                            case 116: // `t' (remove, blacklist temporarily)
                                this.doBlacklisting(true);
                                event.stopPropagation();
                                break;
                        }
                }
        }
    }

    this.switchWithAnchorAncestor = function()
    {
        if (!/^(abbr|acronym|b|basefont|bdo|big|br|button|cite|code|del|dfn|em|font|i|img|ins|kbd|label|map|q|samp|select|small|span|strong|sub|sup|textarea|tt|u|var)$/i.test(this.element.localName)) return; // lagal children of anchors

        var xValue = yarip.calculateXPath(this.element, true);
        var expr = xValue.match(/^.*?\/a\b[^/]*/);
        if (!expr) return;

        var elements = yarip.getElementsByXPath(this.doc, expr[0]);
        if (!elements) return;

        this.element = elements.snapshotItem(0);
        this.xValue = yarip.calculateXPath(this.element);
    }

    this.select = function(handler, noBackup)
    {
        if (!handler) handler = this;

        if (!handler.element) return;

        if (!noBackup) handler.outline = handler.element.style.outline;
        if (handler.element.getAttribute("status") == "whitelisted") handler.element.style.outline = "3px solid #990000";
        else handler.element.style.outline = "3px solid #000099";

        if (!handler.xValue) handler.xValue = yarip.calculateXPath(handler.element);
        if (handler.doc && handler.doc.defaultView) handler.doc.defaultView.status = handler.xValue;
    }

    this.doWhitelisting = function(temporarily)
    {
        var xValue = yarip.calculateXPath(this.element);
        if (!xValue) return;

        if (temporarily)
        {
            if (yarip.whitelistXpath(this.doc, this.address, this.xValue + "/descendant-or-self::*"))
            {
                this.hasTmpWhitelist = true;
                this.select(null, true);
            }
        }
        else if (yarip.whitelistXpath(this.doc, this.address, this.xValue + "/descendant-or-self::*", true, null, true))
        {
            this.hasTmpBlacklist = true; // don't removeAllExceptWhitelisted!
            this.changesMade = true;
            this.select(null, true);
        }
    }

    this.doBlacklisting = function(temporarily)
    {
        if (!this.element) return;

        if (!this.xValue) this.xValue = yarip.calculateXPath(this.element);

        if (temporarily)
        {
            if (yarip.blacklistXpath(this.doc, this.address, this.xValue, false, true))
            {
                this.hasTmpBlacklist = true;
            }
        }
        else if (yarip.blacklistXpath(this.doc, this.address, this.xValue, true, null, null, true))
        {
            this.hasTmpBlacklist = true; // don't removeAllExceptWhitelisted!
            this.changesMade = true;
        }

        this.reset();
    }

    this.reset = function()
    {
        if (this.timeout) clearTimeout(this.timeout);
        this.timeout = null;

        if (this.element) {
            this.element.style.outline = this.outline;
            if (this.element.getAttribute("style") == "") this.element.removeAttribute("style");
        }

        this.element = null;
        this.xValue = null;
        this.outline = null;

        content.document.defaultView.status = "";
    }
}
