
/*
Copyright 2007-2009 Kim A. Brandt <kimabrandt@gmx.de>

This file is part of yarip.

Yarip is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Yarip is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with yarip; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

function YaripList(name, exclusive)
{
    this.id = -1;
    this.name = "";
    this.obj = {};
    this.length = 0;
    this.exclusive = false;
    this.lastChanged = -1;

    this.getId = function()
    {
        return this.id;
    }

    this.setName = function(value)
    {
        if (value) this.name = "" + value;
    }

    this.setExclusive = function(value)
    {
        this.exclusive = "" + value == "true";
    }

    this.getExclusive = function()
    {
        return this.length > 0 && this.exclusive;
    }

    this.setLastChanged = function(value)
    {
        if (value) this.lastChanged = value;
    }

    this.add = function(o, purge)
    {
        if (!o) return;
        var oldObj = this.obj[o.getKey()];
        if (oldObj) {
            oldObj.merge(o);
            if (purge) oldObj.purge();
        } else {
            this.length++;
            if (purge) o.purge();
            this.obj[o.getKey()] = o;
        }
    }

    this.remove = function(value)
    {
        if (!value || !this.obj[value]) return;
        delete this.obj[value];
        this.length--;
    }

    this.reset = function()
    {
        this.obj = {};
        this.length = 0;
    }

    this.sort = function(purge)
    {
        var a = [];
        for (var k in this.obj) {
            a.push(this.obj[k]);
        }
        a.sort(this.sortBy);
        this.reset();
        for (var i = 0; i < a.length; i++) {
            this.add(a[i], purge);
        }
    }

    this.clone = function(purge)
    {
        var tmp = new this.constructor(this.name, this.exclusive);
        for (var k in this) {
            if (this[k] && typeof(this[k]) != "object" && !/^(name|length|exclusive)$/.test(k)) tmp[k] = this[k];
        }
        for (var k in this.obj) {
            tmp.add(this.obj[k].clone(purge));
        }
        return tmp;
    }

    this.merge = function(o)
    {
        this.setExclusive(this.getExclusive() || o.getExclusive());
        for (var k in o.obj) {
            this.add(o.obj[k].clone());
//            this.add(o.obj[k]);
        }
    }

    this.subtract = function(o)
    {
        this.setExclusive(this.getExclusive() && !o.getExclusive());
        for (var k in this.obj) {
            if (o.obj[k]) this.remove(k);
        }
    }

    this.generateXml = function()
    {
        var tmp = "";
        for (var k in this.obj) {
            tmp += this.obj[k].generateXml();
        }
        if (tmp != "") {
            return "\t\t" +
                "<" + this.name + (this.exclusive ? " exclusive=\"" + this.exclusive + "\"" : "") + ">\n" +
                tmp +
                "\t\t</" + this.name + ">\n";
        } else return "";
    }

    this.sortBy = function(a, b)
    {
        if (a.getKey() < b.getKey()) return -1;
        if (a.getKey() > b.getKey()) return 1;
        return 0;
    }

    this.setName(name);
    this.setExclusive(exclusive);
}

function YaripElementList(name, exclusive)
{
    this.id = -1;
    this.name = "";
    this.obj = {};
    this.length = 0;
    this.exclusive = false;

    this.setName(name);
    this.setExclusive(exclusive);
}
YaripElementList.prototype = new YaripList;
YaripElementList.prototype.reduce = function()
{
    var a = [];
    for (var k in this.obj) a.push(this.obj[k]);
    this.reset();

    var r = [];
    var t = [];
    var o = a.shift();
    while (o)
    {
        r.push(o);
//        var noCheck = !/^\/?(\/([a-z][\w]*|\*)(\[@[a-z][\w]*='[^']*'(\ and\ )?\])*)+$/i.test(o.getXPath());
        var noCheck = !/^\/?(\/[a-z]\w*(\[(\d+|@[a-z]\w*='[^']*?'(\sand\s)?)+\])*)+$/i.test(o.getXPath());
        for (var i = 0; i < a.length; i++)
        {
            if (noCheck                                                // do not check
                || !/^\/?(\/[a-z]\w*(\[(\d+|@[a-z]\w*='[^']*?'(\sand\s)?)+\])*)+$/i.test(a[i].getXPath()) // wasn't auto generated
                || a[i].getXPath().indexOf(o.getXPath()) != 0          // does not contain
                || a[i].getXPath().charAt(o.getXPath().length) != "/") // is not the end of a sub-path
            {
                t.push(a[i]);
            }
        }
        a = t;
        t = [];
        o = a.shift();
    }
    a = r;

    for (var i = 0; i < a.length; i++) this.add(a[i]);
}
YaripElementList.prototype.generateXml = function()
{
    var tmp = "";
    for (var k in this.obj) {
        tmp += this.obj[k].generateXml();
    }
    if (tmp != "") {
        return "\t\t" +
            "<" + this.name + (this.exclusive ? " exclusive=\"" + this.exclusive + "\"" : "") + ">\n" +
            tmp +
            "\t\t</" + this.name + ">\n";
    } else return "";
}

function YaripElementWhitelist(name, exclusive)
{
    this.id = 0;
    this.name = "elementWhitelist";
    this.obj = {};
    this.length = 0;
    this.exclusive = false;

    this.setName(name);
    this.setExclusive(exclusive);
}
YaripElementWhitelist.prototype = new YaripElementList;
//YaripElementWhitelist.prototype.set = function(row, col, value)
//{
//    var i = 0;
//    for (var k in this.obj)
//    {
//        if (i++ == row)
//        {
//            var o = this.obj[k];
//            switch (col)
//            {
//                case 0:
//                    if (!yarip.checkXPath(value)) break;

//                    var fh = Components.classes["@mozilla.org/satchel/form-history;1"].getService(Components.interfaces.nsIFormHistory2);
//                    fh.removeEntry("xpath", k);

//                    o.setXPath(value);
//                    this.remove(k);
//                    this.add(o);

//                    fh.addEntry("xpath", value);

//                    this.sort();
//                    yarip.setChangeType(yarip.ESSENTIAL_CHANGE);
//                    break;
//                case 1:
//                    o.setDeep(value);
//                    yarip.setChangeType(yarip.ESSENTIAL_CHANGE);
//                    break;
//            }
//        }
//    }
//}
YaripElementWhitelist.prototype.set = function(row, col, value)
{
    if (this.obj[value]) return;

    var i = 0;
    for (var k in this.obj)
    {
        if (i++ != row) continue;

        var o = this.obj[k];
        switch (col)
        {
            case 0:
                if (!yarip.checkXPath(value)) break;

                var fh = Components.classes["@mozilla.org/satchel/form-history;1"].getService(Components.interfaces.nsIFormHistory2);
                fh.removeEntry("xpath", k);

                o.setXPath(value);
                this.remove(k);
                this.add(o);

                fh.addEntry("xpath", value);

                this.sort();
                yarip.setChangeType(yarip.ESSENTIAL_CHANGE);
                break;
            case 1:
                o.setDeep(value);
                yarip.setChangeType(yarip.ESSENTIAL_CHANGE);
                break;
        }
    }
}
YaripElementWhitelist.prototype.get = function(row, col)
{
    var i = 0;
    for (var k in this.obj)
    {
        if (i++ != row) continue;

        var o = this.obj[k];
        switch (col)
        {
            case -1: return o.getKey();
            case 0: return o.getXPath();
            case 1: return o.getDeep();
            case 2:
                var ms = o.getLastFound();
//                    if (ms > -1) return new Date(ms);
                if (ms > -1) {
                    var date = new Date(ms);
                    return date.toDateString() + " " + date.toLocaleTimeString();
                } else return ""
            case 3: return o.getFound();
            case 4: return o.getNotFound();
            case 5: return o.getIgnored();
            default: return "";
        }
    }
    return "";
}
YaripElementWhitelist.prototype.generateXml = function()
{
    var tmp = "";
    for (var k in this.obj) {
        tmp += this.obj[k].generateXml();
    }
    if (tmp != "") {
        return "\t\t" +
            "<" + this.name + (this.exclusive ? " exclusive=\"" + this.exclusive + "\"" : "") + ">\n" +
            tmp +
            "\t\t</" + this.name + ">\n";
    } else return "";
}

function YaripElementBlacklist(name, exclusive)
{
    this.id = 1;
    this.name = "elementBlacklist";
    this.obj = {};
    this.length = 0;
    this.exclusive = false;

    this.setName(name);
    this.setExclusive(exclusive);
}
YaripElementBlacklist.prototype = new YaripElementList;
//YaripElementBlacklist.prototype.set = function(row, col, value)
//{
//    var i = 0;
//    for (var k in this.obj)
//    {
//        if (i++ == row)
//        {
//            var o = this.obj[k];
//            switch (col)
//            {
//                case 0:
//                    if (!yarip.checkXPath(value)) break;

//                    var fh = Components.classes["@mozilla.org/satchel/form-history;1"].getService(Components.interfaces.nsIFormHistory2);
//                    fh.removeEntry("xpath", k);

//                    o.setXPath(value);
//                    this.remove(k);
//                    this.add(o);

//                    fh.addEntry("xpath", value);

//                    this.sort();
//                    yarip.setChangeType(yarip.ESSENTIAL_CHANGE);
//                    break;
//                case 1:
//                    o.setDeep(value);
//                    yarip.setChangeType(yarip.ESSENTIAL_CHANGE);
//                    break;
//                case 2:
//                    o.setForce(value);
//                    yarip.setChangeType(yarip.ESSENTIAL_CHANGE);
//                    break;
//                case 3:
//                    o.setDOW(value);
//                    yarip.setChangeType(yarip.ESSENTIAL_CHANGE);
//                    break;
//            }
//        }
//    }
//}
YaripElementBlacklist.prototype.set = function(row, col, value)
{
    if (this.obj[value]) return;

    var i = 0;
    for (var k in this.obj)
    {
        if (i++ != row) continue;

        var o = this.obj[k];
        switch (col)
        {
            case 0:
                if (!yarip.checkXPath(value)) break;

                var fh = Components.classes["@mozilla.org/satchel/form-history;1"].getService(Components.interfaces.nsIFormHistory2);
                fh.removeEntry("xpath", k);

                o.setXPath(value);
                this.remove(k);
                this.add(o);

                fh.addEntry("xpath", value);

                this.sort();
                yarip.setChangeType(yarip.ESSENTIAL_CHANGE);
                break;
            case 1:
                o.setDeep(value);
                yarip.setChangeType(yarip.ESSENTIAL_CHANGE);
                break;
            case 2:
                o.setForce(value);
                yarip.setChangeType(yarip.ESSENTIAL_CHANGE);
                break;
            case 3:
                o.setDOW(value);
                yarip.setChangeType(yarip.ESSENTIAL_CHANGE);
                break;
        }
    }
}
YaripElementBlacklist.prototype.get = function(row, col)
{
    var i = 0;
    for (var k in this.obj)
    {
        if (i++ != row) continue;

        var o = this.obj[k];
        switch (col)
        {
            case -1: return o.getKey();
            case 0: return o.getXPath();
            case 1: return o.getDeep();
            case 2: return o.getForce();
            case 3: return o.getDOW();
            case 4:
                var ms = o.getLastFound();
//                    if (ms > -1) return new Date(ms);
                if (ms > -1) {
                    var date = new Date(ms);
                    return date.toDateString() + " " + date.toLocaleTimeString();
                } else return ""
            case 5: return o.getFound();
            case 6: return o.getNotFound();
            case 7: return o.getIgnored();
            default: return "";
        }
    }
    return "";
}
YaripElementBlacklist.prototype.generateCSS = function(notDeep)
{
    var tmp = "";
    for (var k in this.obj) tmp += this.obj[k].generateCSS(notDeep);
    if (tmp != "") return tmp.replace(/,(\n)$/, "$1") + "\t{\n\t\tdisplay: none !important;\n\t}\n";
    else return "";
}
YaripElementBlacklist.prototype.generateXml = function()
{
    var tmp = "";
    for (var k in this.obj) {
        tmp += this.obj[k].generateXml();
    }
    if (tmp != "") {
        return "\t\t" +
            "<" + this.name + (this.exclusive ? " exclusive=\"" + this.exclusive + "\"" : "") + ">\n" +
            tmp +
            "\t\t</" + this.name + ">\n";
    } else return "";
}

function YaripContentWhitelist(name, exclusive)
{
    this.id = 2;
    this.name = "contentWhitelist";
    this.obj = {};
    this.length = 0;
    this.exclusive = false;

    this.setName(name);
    this.setExclusive(exclusive);
}
YaripContentWhitelist.prototype = new YaripList;
YaripContentWhitelist.prototype.set = function(row, col, value)
{
    if (this.obj[value]) return;

    var i = 0;
    for (var k in this.obj)
    {
        if (i++ != row) continue;

        var o = this.obj[k];
        switch (col)
        {
            case 0:
                if (!yarip.checkRegExp(value)) break;

                var fh = Components.classes["@mozilla.org/satchel/form-history;1"].getService(Components.interfaces.nsIFormHistory2);
                fh.removeEntry("regexp", k);

                o.setRegExp(value);
                this.remove(k);
                this.add(o);

                fh.addEntry("regexp", value);

                this.sort();
                yarip.setChangeType(yarip.ESSENTIAL_CHANGE);
                break;
            case 1:
                o.setDeep(value);
                yarip.setChangeType(yarip.ESSENTIAL_CHANGE);
                break;
        }
    }
}
YaripContentWhitelist.prototype.get = function(row, col)
{
    var i = 0;
    for (var k in this.obj)
    {
        if (i++ != row) continue;

        var o = this.obj[k];
        switch (col)
        {
            case -1: return o.getKey();
            case 0: return o.getRegExp();
            case 1: return o.getDeep();
            case 2:
                var ms = o.getLastFound();
                if (ms > -1) {
                    var date = new Date(ms);
                    return date.toDateString() + " " + date.toLocaleTimeString();
                } else return ""
            default: return "";
        }
    }
    return "";
}
YaripContentWhitelist.prototype.generateXml = function()
{
    var tmp = "";
    for (var k in this.obj) {
        tmp += this.obj[k].generateXml();
    }
    if (tmp != "") {
        return "\t\t" +
            "<" + this.name + (this.exclusive ? " exclusive=\"" + this.exclusive + "\"" : "") + ">\n" +
            tmp +
            "\t\t</" + this.name + ">\n";
    } else return "";
}

function YaripContentBlacklist(name, exclusive)
{
    this.id = 3;
    this.name = "contentBlacklist";
    this.obj = {};
    this.length = 0;
    this.exclusive = false;

    this.setName(name);
    this.setExclusive(exclusive);
}
YaripContentBlacklist.prototype = new YaripList;
YaripContentBlacklist.prototype.set = function(row, col, value)
{
    if (this.obj[value]) return;

    var i = 0;
    for (var k in this.obj)
    {
        if (i++ != row) continue;

        var o = this.obj[k];
        switch (col)
        {
            case 0:
                if (!yarip.checkRegExp(value)) break;

                var fh = Components.classes["@mozilla.org/satchel/form-history;1"].getService(Components.interfaces.nsIFormHistory2);
                fh.removeEntry("regexp", k);

                o.setRegExp(value);
                this.remove(k);
                this.add(o);

                fh.addEntry("regexp", value);

                this.sort();
                yarip.setChangeType(yarip.ESSENTIAL_CHANGE);
                break;
            case 1:
                o.setDeep(value);
                yarip.setChangeType(yarip.ESSENTIAL_CHANGE);
                break;
            case 2:
                o.setForce(value);
                yarip.setChangeType(yarip.ESSENTIAL_CHANGE);
                break;
        }
    }
}
YaripContentBlacklist.prototype.get = function(row, col)
{
    var i = 0;
    for (var k in this.obj)
    {
        if (i++ != row) continue;

        var o = this.obj[k];
        switch (col)
        {
            case -1: return o.getKey();
            case 0: return o.getRegExp();
            case 1: return o.getDeep();
            case 2: return o.getForce();
            case 3:
                var ms = o.getLastFound();
                if (ms > -1) {
                    var date = new Date(ms);
                    return date.toDateString() + " " + date.toLocaleTimeString();
                } else return ""
            case 4: return o.getIgnored();
            default: return "";
        }
    }
    return "";
}
YaripContentBlacklist.prototype.generateXml = function()
{
    var tmp = "";
    for (var k in this.obj) {
        tmp += this.obj[k].generateXml();
    }
    if (tmp != "") {
        return "\t\t" +
            "<" + this.name + (this.exclusive ? " exclusive=\"" + this.exclusive + "\"" : "") + ">\n" +
            tmp +
            "\t\t</" + this.name + ">\n";
    } else return "";
}

function YaripAttributesList(name, exclusive)
{
    this.id = 4;
    this.name = "attributes";
    this.obj = {};
    this.length = 0;
    this.exclusive = false;

    this.setName(name);
    this.setExclusive(exclusive);
}
YaripAttributesList.prototype = new YaripList;
YaripAttributesList.prototype.set = function(row, col, value)
{
    if (this.obj[value]) return;

    var i = 0;
    for (var k in this.obj)
    {
        if (i++ != row) continue;

        var o = this.obj[k];
        switch (col)
        {
            case 0:
                if (!yarip.checkXPath(value)) break;

                var fh = Components.classes["@mozilla.org/satchel/form-history;1"].getService(Components.interfaces.nsIFormHistory2);
                fh.removeEntry("xpath", k);

                o.setXPath(value);
                this.remove(k);
                this.add(o);

                fh.addEntry("xpath", value);

                this.sort();
                yarip.setChangeType(yarip.ESSENTIAL_CHANGE);
                break;
            case 1:
                var fh = Components.classes["@mozilla.org/satchel/form-history;1"].getService(Components.interfaces.nsIFormHistory2);
                fh.removeEntry("attribute_name", o.getName());

                o.setName(value);

                fh.addEntry("attribute_name", value);
                yarip.setChangeType(yarip.ESSENTIAL_CHANGE);
                break;
            case 2:
                var fh = Components.classes["@mozilla.org/satchel/form-history;1"].getService(Components.interfaces.nsIFormHistory2);
                fh.removeEntry("attribute_value", o.getValue());

                o.setValue(value);

                fh.addEntry("attribute_value", value);
                yarip.setChangeType(yarip.ESSENTIAL_CHANGE);
                break;
            case 3:
                o.setDeep(value, true);
                yarip.setChangeType(yarip.ESSENTIAL_CHANGE);
                break;
        }
    }
}
YaripAttributesList.prototype.get = function(row, col)
{
    var i = 0;
    for (var k in this.obj)
    {
        if (i++ != row) continue;

        var o = this.obj[k];
        switch (col)
        {
            case -1: return o.getKey();
            case 0: return o.getXPath();
            case 1: return o.getName();
            case 2: return o.getValue();
            case 3: return o.getDeep();
            case 4:
                var ms = o.getLastFound();
//                    if (ms > -1) return new Date(ms);
                if (ms > -1) {
                    var date = new Date(ms);
                    return date.toDateString() + " " + date.toLocaleTimeString();
                } else return ""
            case 5: return o.getFound();
            case 6: return o.getNotFound();
            default: return "";
        }
    }
    return "";
}
YaripAttributesList.prototype.generateXml = function()
{
    var tmp = "";
    for (var k in this.obj) {
        tmp += this.obj[k].generateXml();
    }
    if (tmp != "") {
        return "\t\t" +
            "<" + this.name + (this.exclusive ? " exclusive=\"" + this.exclusive + "\"" : "") + ">\n" +
            tmp +
            "\t\t</" + this.name + ">\n";
    } else return "";
}

function YaripScriptList(name, exclusive)
{
    this.id = 5;
    this.name = "script";
    this.obj = "";
    this.length = 0
    this.exclusive = false;
    this.lastChanged = -1;

    this.setName(name);
    this.setExclusive(exclusive);
}
YaripScriptList.prototype = new YaripList;
YaripScriptList.prototype.set = function(row, col, value)
{
    if (value && value != "")
    {
        this.obj = value;
        this.length = 1;
        this.lastChanged = new Date().getTime();
        yarip.setChangeType(yarip.ESSENTIAL_CHANGE);
    } else {
        this.obj = "";
        this.length = 0;
        this.lastChanged = -1;
        yarip.setChangeType(yarip.ESSENTIAL_CHANGE);
    }
}
YaripScriptList.prototype.get = function(row, col)
{
    return this.obj;
}
YaripScriptList.prototype.clone = function()
{
    var tmp = new this.constructor(this.name);
    for (var k in this) if (this[k] && typeof(this[k]) != "object" && !/^(name|obj|length)$/.test(k)) tmp[k] = this[k];
//    for (var k in this.obj) tmp.add(this.obj[k].clone(purge));
    tmp.set(null, null, this.get());
    return tmp;
}
YaripScriptList.prototype.merge = function(o)
{
//    this.get() == "" ? this.set(null, null, o.get()) : this.set(null, null, this.get() + "\n\n" + o.get());
    var thisScript = this.get();
    var oScript = o.get();
//    if (thisScript != "") {
//        if (oScript != "") {
//            this.set(null, null, thisScript + "\n\n" + oScript);
//        } else {
//            this.set(null, null, thisScript);
//        }
//    } else if (oScript != "") {
//        this.set(null, null, oScript);
//    } else {
//        this.set(null, null, "");
//    }
    if (o.lastChanged > this.lastChanged) {
        this.set(null, null, o.get());
    }
}
YaripScriptList.prototype.subtract = function(o)
{
}
YaripScriptList.prototype.generateXml = function()
{
    if (this.length > 0) {
        return "\t\t<" + this.name + (this.lastChanged > -1 ? " lastChanged=\"" + this.lastChanged + "\"" : "") + "><![CDATA[" + this.get() + "]]></" + this.name + ">\n";
    } else return "";
}

function YaripExtList(name, exclusive)
{
    this.id = 6;
    this.name = "extension";
    this.obj = {};
    this.length = 0;
    this.exclusive = false;

    this.setName(name);
    this.setExclusive(exclusive);
}
YaripExtList.prototype = new YaripList;
YaripExtList.prototype.set = function(row, col, value)
{
    if (this.obj[value]) return;

    var i = 0;
    for (var k in this.obj)
    {
        if (i++ != row) continue;

        var o = this.obj[k];
        switch (col)
        {
            case 1:
                o.setDoElements(value);
                yarip.setChangeType(yarip.ESSENTIAL_CHANGE);
                break;
            case 2:
                o.setDoContents(value);
                yarip.setChangeType(yarip.ESSENTIAL_CHANGE);
                break;
            case 3:
                o.setDoScripts(value);
                yarip.setChangeType(yarip.ESSENTIAL_CHANGE);
                break;
        }
    }
}
YaripExtList.prototype.get = function(row, col)
{
    var i = 0;
    for (var k in this.obj)
    {
        if (i++ != row) continue;

        var o = this.obj[k];
        switch (col)
        {
            case -1: return o.getKey();
            case 0: return yarip.map.objId[k].getName();
            case 1: return o.getDoElements();
            case 2: return o.getDoContents();
            case 3: return o.getDoScripts();
            default: return "";
        }
    }
    return "";
}
YaripExtList.prototype.toArray = function()
{
    var a = [];
    for (var k in this.obj) a[a.length] = [k];
    return a;
}
YaripExtList.prototype.generateXml = function()
{
    var tmp = "";
    for (var k in this.obj) {
        tmp += this.obj[k].generateXml();
    }
    if (tmp != "") {
        return "\t\t" +
            "<" + this.name + (this.exclusive ? " exclusive=\"" + this.exclusive + "\"" : "") + ">\n" +
            tmp +
            "\t\t</" + this.name + ">\n";
    } else return "";
}
