
/*
Copyright 2007-2009 Kim A. Brandt <kimabrandt@gmx.de>

This file is part of yarip.

Yarip is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Yarip is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with yarip; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

function YaripMonitorDialog()
{
    this.id = -1;
    this.hidden = true;
    this.gBrowser = null;
    this.stringbundle = null;
    this.tree = null;
    this.doScroll = true;
    this.known = {}

    this.openAddress = function()
    {
        if (this.tree.currentIndex < 0) return;

        var item = yaripMonitorTreeView.getItem(this.tree.currentIndex);
        var contentLocation = item.content;
        if (!contentLocation) return;

        if (this.gBrowser) this.gBrowser.selectedTab = this.gBrowser.addTab(contentLocation.spec);
    }

    this.copyAddress = function()
    {
        if (this.tree.currentIndex < 0) return;

        var item = yaripMonitorTreeView.getItem(this.tree.currentIndex);
        var contentLocation = item.content;
        if (!contentLocation) return;

        var clipboardHelper = Components.classes["@mozilla.org/widget/clipboardhelper;1"].getService(Components.interfaces.nsIClipboardHelper);
        clipboardHelper.copyString(contentLocation.spec);
    }

    this.whitelistContent = function()
    {
        if (this.tree.currentIndex < 0) return;

        var item = yaripMonitorTreeView.getItem(this.tree.currentIndex);
//        var address = item.address;
        var address = yarip.getAddress(item.location);
        if (!address) return;

        var regExp = yarip.generateRegExp(item.content.spec);
        if (!regExp) return;

        var obj = {
            content: new YaripWLContent(regExp),
            pageAddress: address,
            itemLocation: item.location,
            itemContent: item.content
        }

        window.openDialog("chrome://yarip/content/whitelistcontentdialog.xul", "whitelistcontentdialog", "chrome,modal,resizable", obj);

        if (!obj.content) return;
        if (!obj.pageAddress || obj.pageAddress == "") return;

        if (yarip.whitelistContent(obj.pageAddress, obj.content.getRegExp(), obj.content, item.location))
        {
            yarip.map.sort(false);
            yarip.setChangeType(yarip.ESSENTIAL_CHANGE);
            yarip.pageDialog.reloadPage(obj.pageAddress);
        }
    }

    this.blacklistContent = function()
    {
        if (this.tree.currentIndex < 0) return;

        var item = yaripMonitorTreeView.getItem(this.tree.currentIndex);
//        var address = item.address;
        var address = yarip.getAddress(item.location);
        if (!address) return;

        var regExp = yarip.generateRegExp(item.content.spec);
        if (!regExp) return;

        var obj = {
            content: new YaripBLContent(regExp),
            pageAddress: address,
            itemLocation: item.location,
            itemContent: item.content
        }

        window.openDialog("chrome://yarip/content/blacklistcontentdialog.xul", "blacklistcontentdialog", "chrome,modal,resizable", obj);

        if (!obj.content) return;
        if (!obj.pageAddress || obj.pageAddress == "") return;

        if (yarip.blacklistContent(obj.pageAddress, obj.content.getRegExp(), obj.content, item.location))
        {
            yarip.map.sort(false);
            yarip.setChangeType(yarip.ESSENTIAL_CHANGE);
            yarip.pageDialog.reloadPage(obj.pageAddress);
        }
    }

    this.extendPage = function()
    {
        if (this.tree.currentIndex < 0) return;

        var item = yaripMonitorTreeView.getItem(this.tree.currentIndex);
        yarip.extendPage(item.location, item.content);
    }

    this.copyRegExp = function()
    {
        if (this.tree.currentIndex < 0) return;

        var item = yaripMonitorTreeView.getItem(this.tree.currentIndex);
        var regExp = yarip.generateRegExp(item.content.spec);
        if (!regExp) return;

        var clipboardHelper = Components.classes["@mozilla.org/widget/clipboardhelper;1"].getService(Components.interfaces.nsIClipboardHelper);
        clipboardHelper.copyString(regExp);
    }

//    this.logContent = function(status, location, mode, contentLocation, date)
    this.logContent = function(status, location, mode, contentLocation, date, contentType)
    {
        var item = {}
        item.location = {}
        item.location.href = location.href;
        item.location.protocol = location.protocol;
        item.location.host = location.host;
        item.location.pathname = location.pathname;
        item.content = {};
        item.content.spec = contentLocation.spec;
        item.content.scheme = contentLocation.scheme;
        try{ item.content.host = contentLocation.host; }catch(e){ item.content.host = ""; }
        item.content.path = contentLocation.path;
        item.mode = mode;
        var hours = date.getHours();
        var minutes = date.getMinutes();
        var seconds = date.getSeconds();
        var milliseconds = date.getMilliseconds();
        item.time = (hours < 10 ? "0" : "") + hours + ":" + (minutes < 10 ? "0" : "") + minutes + ":" + (seconds < 10 ? "0" : "") + seconds + "." + (milliseconds < 10 ? "00" : (milliseconds < 100 ? "0" : "")) + milliseconds;
        item.status = status;
        item.page = location.href;
        item.contentLocation = "" + contentLocation.spec;
//        item.contentType = null;
//        item.contentType = contentType ? contentType : "";
        if (status == 2) {
            item.contentType = null;
        } else if (this.known[item.contentLocation]) {
            item.contentType = this.known[item.contentLocation];
        } else if (contentType) {
            item.contentType = contentType;
            this.known[item.contentLocation] = contentType; // is mimeTypeGuess good enough?
        } else {
            item.contentType = null;
        }
        item.statusCode = "";
        yaripMonitorTreeView.appendItem(item, this.doScroll);
    }

    this.updateContentType = function(url, contentType, statusCode)
    {
//        if (!url || !contentType) return;
        if (!url) return;
//        if (this.known[url] == contentType) return;

        this.known[url] = contentType;
        yaripMonitorTreeView.updateItem(url, contentType, statusCode);
//        yaripMonitorTreeView.updateItem("" + url, "" + contentType, "" + statusCode);
//        this.known[url] = contentType;
    }

    this.clear = function()
    {
        yaripMonitorTreeView.clearItems();
        this.known = {}
    }

    this.handleEvent = function(event)
    {
        if (!event) return;

        switch(event.target.id)
        {
            case "yarip-content-tree":
                this.doScroll = yaripMonitorTreeView.rowCount == this.tree.currentIndex + 1;
//                this.doScroll = yaripMonitorTreeView.rowCount > this.tree.currentIndex;
        }
    }

    this.load = function()
    {
        this.stringbundle = document.getElementById("monitor-dialog-stringbundle");
        this.tree = document.getElementById("yarip-content-tree");
        this.tree.view = yaripMonitorTreeView;
        this.tree.addEventListener("select", this, false);
    }

    this.unload = function()
    {
        this.hide();
    }

    this.show = function(gBrowser)
    {
        if (this.hidden)
        {
            this.gBrowser = gBrowser;
            document.getElementById('yarip-monitor-splitter').hidden = false;
            document.getElementById('yarip-monitor').hidden = false;
//            yarip.monitorDialog = this;
            yarip.addMonitorDialog(this);
            this.hidden = false;
        }
    }

    this.hide = function()
    {
        if (!this.hidden)
        {
            document.getElementById('yarip-monitor-splitter').hidden = true;
            document.getElementById('yarip-monitor').hidden = true;
//            yarip.monitorDialog = null;
            yarip.removeMonitorDialog(this);
            this.hidden = true;
        }
    }
}

var yaripMonitorTreeView = {
    treebox: null,
    childData: [],
    whitelisted: Components.classes["@mozilla.org/atom-service;1"].getService(Components.interfaces.nsIAtomService).getAtom("whitelisted"),
    blacklisted: Components.classes["@mozilla.org/atom-service;1"].getService(Components.interfaces.nsIAtomService).getAtom("blacklisted"),

    get rowCount() { return this.childData.length; },

    cycleHeader: function(col) {},
    getCellProperties: function(row, col, properties)
    {
        if (col.id != "status") return;
        if (row < 0 || row >= this.childData.length) return;

        switch (this.childData[row].status)
        {
            case 1: properties.AppendElement(this.whitelisted); break;
            case 2: properties.AppendElement(this.blacklisted); break;
        }
    },
    getCellText: function(row, col)
    {
        var colid = col.id;
        if (colid == "status") return "";
        if (row < 0 || row >= this.childData.length) return "";

        var item = this.childData[row];
        return colid in item ? item[colid] : null;
    },
    canDrop: function(index, orientation) { return false; },
    cycleCell: function(row, col) {},
    drop: function(row, orientation) { return 0; },
    getCellValue: function(row, col) { return ""; },
    getColumnProperties: function(col, properties) {},
    getImageSrc: function(row, col) { return ""; },
    getLevel: function(index) { return 0; },
    getParentIndex: function(rowIndex) { return -1; },
    getProgressMode: function(row, col) { return false; },
    getRowProperties: function(index, properties) {},
    hasNextSibling: function(rowIndex, afterIndex) { return false; },
//    hasNextSibling: function(rowIndex, afterIndex) { return rowIndex < afterIndex && this.childData.length >= afterIndex; },
    isContainerEmpty: function(index) { return false; },
    isContainer: function(index) { return false; },
    isContainerOpen: function(index) { return false; },
    isEditable: function(row, col) { return false; },
    isSelectable: function(row, col) { return false; },
    isSeparator: function(index) { return false; },
    isSorted: function() { return false; },
    performAction: function(action) {},
    performActionOnCell: function(action, row, col) {},
    performActionOnRow: function(action, row) {},
    selectionChanged: function() {},
    setCellText: function(row, col, value) {},
    setCellValue: function(row, col, value) {},
    setTree: function(treebox) { this.treebox = treebox; },
    toggleOpenState: function(index) {},

    appendItem: function(item, doScroll)
    {
        this.childData.push(item);
//        this.childData[this.childData.length] = item;
        if (this.childDataObj[item.contentLocation]) {
            this.childDataObj[item.contentLocation].push(item);
        } else {
            this.childDataObj[item.contentLocation] = [item];
        }

//        var index = this.visibleData.length;
//        this.visibleData.splice(index, 0, item);
//        this.treebox.rowCountChanged(index, 1);
        var index = this.childData.length - 1;
        item.index = index;
        this.treebox.rowCountChanged(index, 1);
        if (doScroll) this.treebox.ensureRowIsVisible(index);
    },

    getItem: function(index)
    {
        return this.childData[index];
    },

//    updateItem: function(url, contentType, statusCode)
//    {
//        var invalidate = false;
//        var _length = this.childData.length - 1;
////        var _min = this.childData.length - 100;
////        for (var i = _length; i >= 0 && i > _min; i--) {
//        for (var i = _length; i >= 0; i--) {
//            var item = this.childData[i];
//            if (item.contentType && item.statusCode) continue; // how good is the mimeTypeGuess?

////            if (item.contentLocation == url) {
//            if (item.contentLocation == "" + url) {
//                if (contentType) {
//                    item.contentType = contentType;
//                    invalidate = true;
//                }
//                if (statusCode) {
//                    item.statusCode = statusCode; // TODO Ensure the right one gets it!
//                    invalidate = true;
//                }
//                if (invalidate) this.treebox.invalidateRow(i);
////                this.treebox.invalidateCell(i, contentType);
////                this.treebox.invalidateCell(i, 4);
////                break;
//            }
//        }
//    },
    childDataObj: {},
    updateItem: function(url, contentType, statusCode)
    {
        var invalidate = false;
        var items = this.childDataObj[url];
        if (!items) {
//            dump("url="+url+", contentType="+contentType+", statusCode="+statusCode+"\n");
            return;
        }

        for (var i = 0; i < items.length; i++) {
            var item = items[i];
            if (item.contentType && item.statusCode) continue; // how good is the mimeTypeGuess?

            if (contentType) {
                item.contentType = contentType;
                invalidate = true;
            }
            if (statusCode) {
                item.statusCode = statusCode; // TODO Ensure the right one gets it!
                invalidate = true;
            }
            if (invalidate) this.treebox.invalidateRow(item.index);
        }
    },

    clearItems: function()
    {
        var oldRows = this.rowCount;
        this.childData = [];
        this.childDataObj = {};
        this.treebox.rowCountChanged(0, -oldRows);
    }
}
