
/*
Copyright 2007-2009 Kim A. Brandt <kimabrandt@gmx.de>

This file is part of yarip.

Yarip is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Yarip is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with yarip; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

function YaripWhitelistDialog()
{

    this.pageMenulist = null;
    this.xpathTextbox = null;
    this.doc = null;
    this.obj = null;
    this.xValue = null;

    this.setXPath = function(value)
    {
        if (!this.obj) return;
        this.obj.element.setXPath(value);
    }

    this.highlight = function()
    {
        if (!this.doc) return;

        if (this.xValue) this.unHighlight(this.doc, this.xValue);
        this.xValue = null;
        var xValue = this.obj.element.getXPath();
        if (!yarip.highlight(this.doc, xValue))
        {
            var msg = this.stringbundle.getString("ERR_INVALID_XPATH");
            alert(msg);
            throw new YaripException(msg);
        }
        this.xValue = xValue;
    }

    this.unHighlight = function(doc, xValue)
    {
        yarip.unHighlight(doc, xValue);
    }

    this.removeIndexes = function()
    {
        var xValue = this.xpathTextbox.value.replace(/\[\d+\]/g, "");
        this.xpathTextbox.value = xValue;
        this.setXPath(xValue);
    }

    this.load = function()
    {
        if (!"arguments" in window || window.arguments.length < 2) return;

        this.doc = window.arguments[0];
        this.obj = window.arguments[1];
        if (/*!this.doc ||*/ !this.obj) return;

        this.stringbundle = document.getElementById("elementWhitelist-dialog-stringbundle");
        this.pageMenulist = document.getElementById("page");
        this.xpathTextbox = document.getElementById("xpath");

        this.pageMenulist.value = this.obj.pageAddress;
        this.xpathTextbox.value = this.obj.element.getXPath();
        this.xpathTextbox.focus();
        this.xpathTextbox.select();

        if (!this.doc) {
            this.pageMenulist.disabled = true;
            document.getElementById("highlight").disabled = true;
            return;
        }

        var menupopup = document.getElementById("page-menupopup");
        var menuitem = null;
        var address = yarip.getAddress(this.doc.location, yarip.PAGE_MODE);
        if (address)
        {
            menuitem = document.createElement("menuitem");
            menuitem.setAttribute("label", address);
            menuitem.setAttribute("value", address);
            menupopup.appendChild(menuitem);
        }
        address = yarip.getAddress(this.doc.location, yarip.FIRSTDIR_MODE);
        if (address)
        {
            menuitem = document.createElement("menuitem");
            menuitem.setAttribute("label", address);
            menuitem.setAttribute("value", address);
            menupopup.appendChild(menuitem);
        }
        address = yarip.getAddress(this.doc.location, yarip.FQDN_MODE);
        if (address)
        {
            menuitem = document.createElement("menuitem");
            menuitem.setAttribute("label", address);
            menuitem.setAttribute("value", address);
            menupopup.appendChild(menuitem);
        }
        address = yarip.getAddress(this.doc.location, yarip.SLD_MODE);
        if (address)
        {
            menuitem = document.createElement("menuitem");
            menuitem.setAttribute("label", address);
            menuitem.setAttribute("value", address);
            menupopup.appendChild(menuitem);
        }
    }

    this.accept = function()
    {
        if (this.doc) {
            this.unHighlight(this.doc, this.xValue);
            if (!this.obj) return;

            this.obj.pageAddress = this.pageMenulist.value;

            var elements = yarip.getElements(this.doc, this.obj.element.getXPath());
            if (!elements)
            {
                var msg = this.stringbundle.getString("ERR_INVALID_XPATH");
                alert(msg);
                throw new YaripException(msg);
            }
        } else {
            if (!this.obj) return;
            if (!yarip.checkXPath(this.obj.element.getXPath()))
            {
                var msg = this.stringbundle.getString("ERR_INVALID_XPATH");
                alert(msg);
                throw new YaripException(msg);
            }
        }

        var fh = Components.classes["@mozilla.org/satchel/form-history;1"].getService(Components.interfaces.nsIFormHistory2);
        fh.addEntry("xpath", this.obj.element.getXPath());

        return;
    }

    this.cancel = function()
    {
        if (this.doc) this.unHighlight(this.doc, this.xValue);
        if (this.obj) this.obj.element = null;
    }
}
